%* lsaf_addrolemember                                                                             *;
%*                                                                                                *;
%* Adds a user account or user group as a member of a role in the specified context.              *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_ADDROLEMEMBER(LSAF_PATH=lsaf-path, LSAF_ROLE=lsaf-role-name,                             *;
%* LSAF_ROLE_CONTEXT=lsaf-role-context, LSAF_MEMBER=lsaf-member-identifier                        *;
%* <, LSAF_TYPE=lsaf-member-type, LSAF_GROUP_CONTEXT=lsaf-group-context>);                        *;
%*                                                                                                *;
%* Example Code                                                                                   *;
%* Context paths used in examples:                                                                *;
%* -  Organization: /SAS                                                                          *;
%* -  Project:      /SAS/Project1                                                                 *;
%* -  Analysis:     /SAS/Project1/Analysis1                                                       *;
%*                                                                                                *;
%* 1. Add a user account to a role that is defined in a project:                                  *;
%*    %lsaf_addrolemember(lsaf_path=%str(/SAS/Project1), lsaf_role=%str(PrjRole1),                *;
%*    lsaf_role_context=%str(/SAS/Project1),lsaf_member=%str(User1), lsaf_type=%str(USER));       *;
%*                                                                                                *;
%* 2. Add a user group that is defined in an organization to a role that is defined in a project: *;
%*    %lsaf_addrolemember(lsaf_path=%str(/SAS/Project1), lsaf_role=%str(PrjRole1),                *;
%*    lsaf_role_context=%str(/SAS/Project1),lsaf_member=%str(OrgGrp1), lsaf_type=%str(GROUP),     *;
%*    lsaf_group_context=%str(/SAS));                                                             *;
%*                                                                                                *;
%* 3. Add a user account to a role that is assigned in a project:                                 *;
%*    %lsaf_addrolemember(lsaf_path=%str(/SAS/Project1), lsaf_role=%str(OrgRole1),                *;
%*    lsaf_role_context=%str(/SAS),lsaf_member=%str(User1), lsaf_type=%str(USER));                *;
%*                                                                                                *;
%* 4. Add a user group that is defined in an organization to a role that is assigned in a project:*;
%*    %lsaf_addrolemember(lsaf_path=%str(/SAS/Project1), lsaf_role=%str(OrgRole1),                *;
%*    lsaf_role_context=%str(/SAS),lsaf_member=%str(OrgGrp1), lsaf_type=%str(GROUP),              *;
%*    lsaf_group_context=%str(/SAS));                                                             *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - Required - The path of the context item in which the role is assigned.      *;
%* @param lsaf_role - Required - The name of the role.                                            *;
%* @param lsaf_role_context - Optional - The context in which the role is defined, specified as a *;
%*            path (such as /SAS). If not provided, the member is added to the role that is       *;
%*            defined at the path.                                                                *;
%* @param lsaf_member - Required - The identifier of the user account or the name of the user     *;
%*            group to add.                                                                       *;
%* @param lsaf_type - Conditional - The type of member to add. Valid values (case-insensitive):   *;
%*            USER (Default) and GROUP.                                                           *;
%* @param lsaf_group_context - conditional - The context in which the group is defined, specified *;
%*            as a path (such as /SAS). This parameter is required, if LSAF_TYPE=GROUP.           *;
%*            Otherwise, it is ignored.                                                           *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.2                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-08-30  updates for standardization and coding conventions                        *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%* @history 2019-01-29  update to 2.1                                                             *;
%*                                                                                                *;

%macro lsaf_addrolemember(
  lsaf_path=,
  lsaf_role=,
  lsaf_role_context=,
  lsaf_member=,
  lsaf_type=USER,
  lsaf_group_context=
  ) /des='Add a member to the role in the specified context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/RoleService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("addMember", "%bquote(&lsaf_path)", "%bquote(&lsaf_role)", "%bquote(&lsaf_role_context)", "%bquote(&lsaf_type)",  "%bquote(&lsaf_member)", "%bquote(&lsaf_group_context)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The &lsaf_type &lsaf_member has been added to &lsaf_role in &lsaf_path.." ;
    end;
      else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot add &lsaf_type &lsaf_member to &lsaf_role in &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_addrolemember;
