%* lsaf_addroleprivilege                                                                         *;
%*                                                                                               *;
%* Adds a privilege to a role in the specified context.                                          *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_ADDROLEPRIVILEGE(LSAF_PATH=lsaf-path, LSAF_ROLE=lsaf-role-name,                         *;
%* LSAF_PRIVILEGE=lsaf-privilege-id);                                                            *;
%*                                                                                               *;
%* Example Code:                                                                                 *;
%* Use %lsaf_getscopedprivileges to fetch the list of privileges that are available at the .     *;
%* context level. The macro returns a SAS data set that contains the list of the privileges.     *;
%* The default name of the data set is work.lsafgetscopedprivileges). Use one of the privilegeId *;
%* values to add that privilege to a role.                                                       *;
%*                                                                                               *;
%* 1. Retrieve the available privileges in a project:                                            *;
%*    %lsaf_getscopedprivileges(lsaf_path=%str(/SAS/Project1));                                  *;
%*                                                                                               *;
%* 2. Add a privilege to a role:                                                                 *;
%*    %lsaf_addroleprivilege(lsaf_path=%str(/SAS/Project1), lsaf_role=%str(PrjRole1),            *;
%*    lsaf_privilege=%str(PRIVILEGE_SIGN_FILE));                                                 *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - Required - The path of the context in which the role is defined.           *;
%* @param lsaf_role - Required - The name of the role.                                           *;
%* @param lsaf_privilege - Required - The identifier of the privilege.                           *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%* @history 2019-01-29  update to 2.1                                                            *;
%*                                                                                               *;

%macro lsaf_addroleprivilege(
  lsaf_path=,
  lsaf_role=,
  lsaf_privilege=
  ) /des='Add a privilege to the role in the specified context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/RoleService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("addPrivilege", "%bquote(&lsaf_path)", "%bquote(&lsaf_role)", "%bquote(&lsaf_privilege)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The privilege &lsaf_privilege has been added to &lsaf_role in &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot add privilege &lsaf_privilege to &lsaf_role in &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_addroleprivilege;
