%* lsaf_createprocessflowmanifest                                                                 *;
%*                                                                                                *;
%* Creates a process flow manifest file in the repository. The SAS macro                          *;
%* variable _lsafCreatedProcessFlowManifest_ is set to the full path of the process flow          *;
%* manifest file, which includes the file extension. If the process flow manifest cannot be       *;
%* created, _lsafCreatedProcessFlowManifest_ is set to blank.                                     *;
%*                                                                                                *;
%* If the manifest file exists, a new version of the file is created. If the existing file is     *;
%* not versioned, it is overwritten by the new file.                                              *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_CREATEPROCESSFLOWMANIFEST(LSAF_PATH=lsaf-path, LSAF_PROCESSFLOW=process-flow-name,       *;
%* LSAF_MANIFESTPATH=manifest-path <, LSAF_ENABLEVERSIONING=lsaf-enable-versioning,               *;
%* LSAF_VERSIONTYPE=lsaf-version-type, LSAF_CUSTOMVERSION=lsaf-custom-version,                    *;
%* LSAF_COMMENT=lsaf-comment>);                                                                   *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%* @macvar _lsafCreatedProcessFlowManifest_ The full path of the process flow manifest.           *;
%*                                                                                                *;
%* @param lsaf_path - Required - The context path where the process flow is located.              *;
%* @param lsaf_processflow - Required - The name of the process flow.                             *;
%* @param lsaf_manifest_path - Required - The repository location for the manifest file. The      *;
%*            location must be a folder and must exist in the repository.                         *;
%* @param lsaf_enableversioning - Optional - Indicates whether the file to create is              *;
%*            versioned. Valid values:                                                            *;
%*            - 0: Create an unversioned file.                                                    *;
%*            - 1 (Default): Create a versioned file.                                             *;
%* @param lsaf_versiontype - Optional - The version type (MINOR, MAJOR, or CUSTOM) to create,     *;
%*            if the file is versioned. The default value is MAJOR. The value is ignored for      *;
%*            unversioned files.                                                                  *;
%* @param lsaf_customversion - Conditional - The version number to assign to the file to create.  *;
%*            The value must be in the valid version label format, which includes a decimal       *;
%*            point. This option is required when versiontype is CUSTOM, otherwise, it is ignored.*;
%* @param lsaf_comment - Optional - The audit trail comment for the manifest file creation.       *;
%*                                                                                                *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.5                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2016-11-15  initial programming                                                       *;
%* @history 2019-02-12  update to 2.1                                                             *;
%* @history 2019-08-28  update file path length to $2048                                          *;
%*                                                                                                *;

%macro lsaf_createprocessflowmanifest(
  lsaf_path=,
  lsaf_processflow=,
  lsaf_manifest_path=,
  lsaf_enableversioning=,
  lsaf_versiontype=,
  lsaf_customversion=,
  lsaf_comment=
  ) /des='Create a manifest file for the specifed process flow in the indicated SAS Life Science Analytics Framework location.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafCreatedProcessFlowManifest_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode     length=8
           message        length=$200
           logMsgPrefix   length=$70
           logMsgPrimary  length=$70
           logMsg         length=$300
           manifestPath   length=$2048
    ;
    manifestPath="";

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ProcessFlowService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("createManifest", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", "%bquote(&lsaf_manifest_path)",
                             "%bquote(&lsaf_enableversioning)", "%bquote(&lsaf_versiontype)", "%bquote(&lsaf_customversion)", "%bquote(&lsaf_comment)", results, manifestPath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot create a manifest for process flow &lsaf_processflow in &lsaf_manifest_path.." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The manifest for process flow &lsaf_processflow was created in &lsaf_manifest_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafCreatedProcessFlowManifest_= " manifestPath;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafCreatedProcessFlowManifest_", manifestPath);

  run;

%mend lsaf_createprocessflowmanifest;
