%* lsaf_createworkspacejob                                                                           *;
%*                                                                                                   *;
%* Creates a job in the workspace of the current user. The SAS macro variable _lsafWorkspaceJobPath_ *;
%* is set to the full path of the created job, which includes the file extension. If the job is not  *;
%* created, _lsafWorkspaceJobPath_ is set to blank.                                                  *;
%*                                                                                                   *;
%* Data sets that represent the job data to be set at creation time can be specified in the call to  *;
%* lsaf_createworkspacejob. If data sets are not specified, the default values for all job data are  *;
%* used.                                                                                             *;
%* The output data sets from the following macros contain all of the data that is relevant to the    *;
%* job properties that can be specified when the job is created. See the macro documentation for     *;
%* more details.                                                                                     *;
%* - lsaf_getjobinfo or lsaf_getworkspacejobinfo                                                     *;
%* - lsaf_getjobprograms or lsaf_getworkspacejobprograms                                             *;
%* - lsaf_getjobparameters or lsaf_getworkspacejobparameters                                         *;
%* - lsaf_getjobinputs or lsaf_getworkspacejobinputs                                                 *;
%* - lsaf_getjoboutputs or lsaf_getworkspacejoboutputs                                               *;
%*                                                                                                   *;
%* The output data sets from the following job manifest macros contain all of the data that is       *;
%* relevant to the job properties that can be specified when the job is created. See the macro       *;
%* documentation for more details.                                                                   *;
%* - lsaf_getjobmanifestinfo                                                                         *;
%* - lsaf_getjobmanifestprograms                                                                     *;
%* - lsaf_getjobmanifestparameters                                                                   *;
%* - lsaf_getjobmanifestinputs                                                                       *;
%* - lsaf_getjobmanifestoutputs                                                                      *;
%*                                                                                                   *;
%* Syntax:                                                                                           *;
%* %LSAF_CREATEWORKSPACEJOB(LSAF_PATH=lsaf-path <SAS_DSNAME_INFO=SAS-data-set-info,                  *;
%*      SAS_DSNAME_PROGRAMS=SAS-data-set-jobprograms, SAS_DSNAME_PARAMETERS=SAS-data-set-parameters, *;
%*      SAS_DSNAME_INPUTS=SAS-data-set-inputs, SAS_DSNAME_OUTPUTS=SAS-data-set-outputs>);            *;
%*                                                                                                   *;
%* @macvar _lsafRC_  The return code.                                                                *;
%* @macvar _lsafMsg_ The return message.                                                             *;
%* @macvar _lsafWorkspaceJobPath_ The full path of the new job.                                      *;
%*                                                                                                   *;
%* @param lsaf_path - Required - The path for the job to create in the workspace of the current user.*;
%* @param sas_dsname_info - Optional - The name of the SAS data set that contains the job            *;
%*            information, specified as libref.dataset. If the parameter is not included in the call,*;
%*            the default job information is set.  The data set must contain at least the            *;
%*            character variables that are listed below. Additional variables are ignored.           *;
%*            - name: The name of the job property.                                                  *;
%*            - value: The value of the property.                                                    *;
%*            The information that can be specified is DESCRIPTION, LOGPATH, LSTPATH, MANIFESTPATH,  *;
%*            NEWOUTPUTS_CUSTOMVERSION, NEWOUTPUTS_ENABLEVERSION, and NEWOUTPUTS_VERSIONTYPE.        *;
%*            All other values listed in the data set are ignored. All data are case-insensitive.    *;
%*            If a name is listed more than once, the last value processed is used. If a name is     *;
%*            omitted, the default value is used.                                                    *;
%* @param sas_dsname_programs - Optional - The name of the SAS data set that contains the list of    *;
%*            paths to the SAS programs to be run, specified as libref.dataset. If the parameter is  *;
%*            not included in the call, no programs are set. The data set must contain at least      *;
%*            the variables that are listed below. Additional variables are ignored.                 *;
%*            - programPath: The path of the SAS program file.                                       *;
%*            - programVersion: The version of the SAS program file. Blank values specify the        *;
%*              latest file version.                                                                 *;
%* @param sas_dsname_parameters - Optional - The name of the SAS data set that contains the job      *;
%*            parameter information, specified as libref.dataset. If the parameter is not included in*;
%*            the call, no parameters are set. The data set must contain at least the character      *;
%*            variables listed below. Additional variables are ignored.                              *;
%*            - name: The name of the parameter.                                                     *;
%*            - label: The label for the parameter.                                                  *;
%*            - type: The parameter type.  Valid values are CHARACTER, CHARACTER_MASKED, NUMERIC,    *;
%*              DATE, FILE, and FOLDER.                                                              *;
%*            - defaultValue: The value to set as the default value for the parameter. DATE job      *;
%*              parameter values must be one of the following formats. Date values in formats other  *;
%*              than the two described below result in an execution error.                           *;
%*              - SAS datetime18. format (such as 02NOV2013:09:33:22) - an offset of zero hours from *;
%*                GMT is used.                                                                       *;
%*              - SAS E8601DZ25. format (such as 2013-11-02T09:33:22-04:00).                         *;
%*            - fileVersion: The file version to use, if the parameter is of type FILE. Blank values *;
%*              specifiy the latest file version.                                                    *;
%*            - includeSubFolders (numeric): Indicates whether the folder includes subfolders.       *;
%*              Valid values: 0 and 1.                                                   *;
%*            Data set rows with duplicate values for parameter name result in a data set            *;
%*            validation error.                                                                      *;
%* @param sas_dsname_inputs - Optional - The name of the SAS data set that contains the list of the  *;
%*            job inputs, specified as libref.dataset. If the parameter is not included in the call, *;
%*            no inputs are set. The data set must contain at least the variables that are listed    *;
%*            below as the type specified. Additional variables are ignored.                         *;
%*            - inputPath (character): The path of the file or container to use as input.            *;
%*            - inputType (character): The type of input item. Valid values: FILE and CONTAINER.     *;
%*            - inputVersion (character): The version of the FILE item. Blank values specify the     *;
%*              latest file version. The value is ignored for inputs of type CONTAINER.              *;
%*            - inputIncludeSubFolders (numeric): Indicates whether the input folder includes        *;
%*              subfolders. Valid values: 0 and 1. The value is ignored for inputs of type FILE.     *;
%*            Data set rows with duplicate values for inputPath result in a data set validation      *;
%*            error.                                                                                 *;
%* @param sas_dsname_outputs - Optional - The name of the SAS data set that contains the list of the *;
%*            job outputs, specified as libref.dataset. If the parameter is not included in the call,*;
%*            no outputs are set. The data set must contain at least the variables listed that are   *;
%*            below as the type specified. Additional variables are ignored.                         *;
%*            - outputPath (character): The path of the container to use as output.                  *;
%*            - outputIncludeSubFolders (numeric): Indicates whether the output container includes   *;
%*              subfolders. Valid values: 0 and 1.                                                   *;
%*            Data set rows with duplicate values for outputPath result in a data set validation     *;
%*            error.                                                                                 *;
%*                                                                                                   *;
%* @since 1.5                                                                                        *;
%* @exposure external                                                                                *;
%*                                                                                                   *;
%* @history 2016-09-23  initial coding                                                               *;
%* @history 2019-02-11  update to 2.1                                                                *;
%* @history 2019-08-28  update file path length to $2048                                             *;
%* @history 2019-12-03  update header documentation to include job manifest macros.                  *;
%*                                                                                                   *;

%macro lsaf_createworkspacejob(
   lsaf_path=,
   sas_dsname_info=%str(_lsaf_createworkspacejob_ds_notprovided_),
   sas_dsname_programs=%str(_lsaf_createworkspacejob_ds_notprovided_),
   sas_dsname_parameters=%str(_lsaf_createworkspacejob_ds_notprovided_),
   sas_dsname_inputs=%str(_lsaf_createworkspacejob_ds_notprovided_),
   sas_dsname_outputs=%str(_lsaf_createworkspacejob_ds_notprovided_)
   ) /des='Create a job in the workspace.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
  %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
           _lsafWorkspaceJobPath_
           notesOption
   ;

   %local haveData
          logMsgPrefix
          logMsgPrimary
          macroName
   ;

   %let haveData =%str(1);
   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly. Unknown err%str(or).);
   %let _lsafWorkspaceJobPath_=;

   %let notesOption=%sysfunc(getoption(notes));

   %*********************************************************************************;
   %* Validate all input data sets and create master data set to set server values. *;
   %*********************************************************************************;
   %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
   %lsaf_process_jobdatasets(callingMacro=&macroName,
                              sas_dsname_info=&sas_dsname_info,
                              sas_dsname_programs=&sas_dsname_programs,
                              sas_dsname_parameters=&sas_dsname_parameters,
                              sas_dsname_inputs=&sas_dsname_inputs,
                              sas_dsname_outputs=&sas_dsname_outputs
   );
   %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
   %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );

   options notes;
   %if (&_lsafRC_ ne 0) %then
   %do;
      %GOTO PRINT_DSERROR_LOG_MSGS;
   %end;
   %if (&_lsafMASTERJobDATA_tempDS_ EQ %str(_null_)) %then
   %do;
      %LET haveData=0;
   %end;

   %let _lsafRC_ =%str(-999);

   %**********************************************;
   %* Process the records                        *;
   %**********************************************;
   data _null_;
      attrib
          returnCode      length=8
          message         length=$200
          jobPath         length=$2048
          logMsgPrefix    length=$70
          logMsgPrimary   length=$70
          logMsg          length=$300
      ;
      jobPath="";

      %if (&haveData EQ 1) %then
      %do;
         set &_lsafMASTERJobDATA_tempDS_ end=eof;
         name=strip(name);

      if (_n_ EQ 1) then
      do;
      %end;
         %***********************************;
         %* Declare the java objects, once  *;
         %***********************************;
         declare javaobj srv("com/sas/lsaf/macro/service/JobService");
         declare javaobj results("com/sas/lsaf/macro/results/Result");
         declare javaobj info("com/sas/lsaf/macro/results/support/SasJobInfo");
         declare javaobj programs("com/sas/lsaf/macro/results/support/SasJobItemSpecifications");
         declare javaobj params("com/sas/lsaf/macro/results/support/SasJobParameters");
         declare javaobj inputs("com/sas/lsaf/macro/results/support/SasJobItemSpecifications");
         declare javaobj outputs("com/sas/lsaf/macro/results/support/SasJobItemSpecifications");
      %if (&haveData EQ 1) %then
      %do;
      end;

      if (source EQ "&source_info") then
      do;
         name=kstrip(lowcase(name));
         select (name);
            when ("description") info.callVoidMethod("setDescription", kstrip(value));
            when ("logpath") info.callVoidMethod("setLogPath", kstrip(value));
            when ("lstpath") info.callVoidMethod("setLstPath", kstrip(value));
            when ("manifestpath") info.callVoidMethod("setManifestPath", kstrip(value));
            when ("newoutputs_customversion") info.callVoidMethod("setCustomVersion", kstrip(value));
            when ("newoutputs_enableversion") info.callVoidMethod("setEnableVersioning", kstrip(value));
            when ("newoutputs_versiontype") info.callVoidMethod("setVersionType", kstrip(upcase(value)));
            otherwise;
         end;
      end;
      else if (source EQ "&source_parms") then
      do;
         params.callVoidMethod("addValue", kstrip(name), kstrip(pLabel), kstrip(type), kstrip(value), kstrip(version), includeSubFolders);
      end;
      else if (source EQ "&source_programs") then
      do;
         programs.callVoidMethod("addProgramValue", kstrip(value), kstrip(version));
      end;
      else if (source EQ "&source_inputs") then
      do;
         inputs.callVoidMethod("addInputValue", kstrip(value), kstrip(version), includeSubFolders, kstrip(type));
      end;
      else if (source EQ "&source_outputs") then
      do;
         outputs.callVoidMethod("addOutputValue", kstrip(value), includeSubFolders);
      end;

      %***********************************;
      %* Attempt the operation           *;
      %***********************************;
      if (eof);
      %end;

      srv.callStaticStringMethod("createWorkspaceJob", "%bquote(&lsaf_path)", info, programs, params, inputs, outputs, results, jobPath);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (returnCode ne 0) then
      do;
         logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
         logMsg=" Could not create job in the workspace: &lsaf_path.";
      end;
      else
      do;
         logMsgPrimary=logMsgPrefix;
         logMsg=" Created job in workspace using path &lsaf_path.";
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrimary " " logMsg;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrefix " _lsafMsg_= " message;
      put logMsgPrefix " _lsafRC_= " returnCode;

      %if (&haveData NE 1) %then
      %do;
         put logMsgPrefix " No job data was provided.";
      %end;
      %else
      %do;
         %if (&sas_dsname_info NE _lsaf_createworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_info = &sas_dsname_info";
         %end;
         %if (&sas_dsname_programs NE _lsaf_createworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_programs = &sas_dsname_programs";
         %end;
         %if (&sas_dsname_parameters NE _lsaf_createworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_parameters = &sas_dsname_parameters";
         %end;
         %if (&sas_dsname_inputs NE _lsaf_createworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_inputs = &sas_dsname_inputs";
         %end;
         %if (&sas_dsname_outputs NE _lsaf_createworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_outputs = &sas_dsname_outputs";
         %end;
      %end;

      put logMsgPrefix " _lsafWorkspaceJobPath_=" jobPath;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);
      call symputx("_lsafWorkspaceJobPath_", kstrip(jobPath));

   run;

   options &notesOption;

   %return;

   %PRINT_DSERROR_LOG_MSGS:
      options &notesOption;
      %put &logMsgPrimary  &macroName ;
      %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
      %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
      %put;

   options &notesOption;

%mend lsaf_createworkspacejob;
