%* lsaf_downloadaszip                                                                             *;
%*                                                                                                *;
%* Downloads a SAS Life Science Analytics Framework file or folder along with its content         *;
%* to a local computer as a ZIP file.                                                             *;
%*                                                                                                *;
%* If the local directory does not exist, it is created. If the local file does not exist,        *;
%* it is created. If the item to download is a folder, the contents of the folder are             *;
%* downloaded and the hierarchy is maintained. If the item to download is a context, the          *;
%* containers (child contexts and folders) and files are downloaded.                              *;
%*                                                                                                *;
%* NOTE: This macro is supported only from a PC SAS session. Using this macro within an active    *;
%* SAS Life Science Analytics Framework application session result in a failure.                  *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_DOWNLOADASZIP(LSAF_PATH=lsaf-path, LOCAL_PATH=local-zipfile-path                         *;
%* <,LSAF_OVERWRITE=lsaf-overwrite> );                                                            *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - Required - The path of the item to download.                                *;
%* @param local_path - Required - The full path and name of the file to create (or overwrite,     *;
%*            if applicable) on the local computer. File extensions are entered on the file       *;
%*            as entered by the user. If no extension is entered, it results in ZIP file          *;
%*            with no extension.                                                                  *;
%* @param lsaf_overwrite - Optional -  Indicates whether to overwrite the existing ZIP file on    *;
%*            the local computer. Valid values:                                                   *;
%*            - 0 (Default): Maintain                                                             *;
%*            - 1: Overwrite                                                                      *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.4                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-08-30  updates for standardization and coding conventions                        *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%* @history 2018-01-09  updates for LSAF 5.1                                                      *;
%*                                                                                                *;

%macro lsaf_downloadaszip(
  lsaf_path=,
  local_path=,
  lsaf_overwrite=0
  ) /des='Downloads a SAS Life Science Analytics Framework repository item as a zip file.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300
    ;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/BatchRepositoryService") ;
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("downloadAsZip", "%bquote(&lsaf_path)", "%bquote(&local_path)", "%bquote(&lsaf_overwrite)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Print messages in log           *;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not attempt the operation to download the item as a zip file: &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Attempted to download the item as a zip file: &lsaf_path.." ;
    end;
    
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;
    
    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_downloadaszip;
