%* lsaf_downloadfile                                                                                 *;
%*                                                                                                   *;
%* Downloads a file to a local computer from the repository.                                         *;
%*                                                                                                   *;
%* If the local folder does not exist, it is created. If the local file exists, it is overwritten.   *;
%*                                                                                                   *;
%* NOTE: This macro is supported only from a PC SAS session. Using this macro within an active       *;
%* SAS Life Science Analytics Framework application session results in a failure.                    *;
%*                                                                                                   *;
%* Syntax:                                                                                           *;
%* %LSAF_DOWNLOADFILE(LSAF_PATH=lsaf-path, LSAF_VERSION=lsaf-version>, LOCAL_PATH=local-path);       *;
%*                                                                                                   *;
%* @macvar _lsafRC_  The return code.                                                                *;
%* @macvar _lsafMsg_ The return message.                                                             *;
%*                                                                                                   *;
%* @param lsaf_path - Required - The path of the file in the repository.                             *;
%* @param lsaf_version - Conditional - The version of the file to download, if the file is           *;
%*            versioned. If the file is versioned and a specific version is specified, the value     *;
%*            must be in the valid format, which includes a decimal point, such as 1.0 or 4.2.       *;
%*            If the file is versioned and no version is specified, the latest version is            *;
%*            downloaded. If the file is unversioned and a specific version is specified, a failure  *;
%*            occurs.                                                                                *;
%* @param local_path - Required - The full path and name of the file on the local computer.          *;
%*                                                                                                   *;
%* @macrotype LSAF                                                                                   *;
%* @since 1.3                                                                                        *;
%* @exposure external                                                                                *;
%*                                                                                                   *;
%* @history 2013-10-30  initial coding                                                               *;
%* @history 2016-08-26  rebrand as LSAF                                                              *;
%* @history 2019-02-16  Update to 2.1                                                                *;
%*                                                                                                   *;

%macro lsaf_downloadfile(
  lsaf_path=,
  lsaf_version=,
  local_path=
  ) /des='Download a file to your local computer from the SAS Life Science Analytics Framework repository.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib logMsgPrimary   length=$70;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("downloadFile", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", "%bquote(&local_path)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      if ("&lsaf_version" eq " ") then
      do;
        logMsg=" Latest version of &lsaf_path. was downloaded successfully to &local_path..";
      end;
      else
      do;
        logMsg=" Version &lsaf_version. of &lsaf_path. was downloaded successfully to &local_path..";
      end;
    end;
    else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" The file was not downloaded: &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_downloadfile;
