%* lsaf_exportextdicts                                                                                  *;
%*                                                                                                      *;
%* Exports the metadata for external dictionaries. The metadata is stored in a SAS data set in the      *;
%* specified location (WORKSPACE or REPOSITORY).                                                        *;
%*                                                                                                      *;
%* Syntax:                                                                                              *;
%* %LSAF_EXPORTEXTDICTS(                                                                                *;
%*     LSAF_LOCATION=lsaf-location,                                                                     *;
%*     LSAF_PATH=lsaf-path,                                                                             *;
%*     LSAF_OVERWRITE=lsaf-overwrite,                                                                   *;
%*     LSAF_ENABLEVERSIONING=lsaf-enableversioning,                                                     *;
%*     LSAF_VERSIONTYPE=lsaf-versiontype,                                                               *;
%*     LSAF_CUSTOMVERSION=lsaf-customversion,                                                           *;
%*     LSAF_COMMENT=lsaf-comment);                                                                      *;
%*                                                                                                      *;
%* @macvar _lsafRC_  The return code.                                                                   *;
%* @macvar _lsafMsg_ The return message.                                                                *;
%* @macvar _lsafExtDictionaryPath_ The full path of the external dictionary data set, which includes    *;
%*     the file extension. If the dictionary data set is not created, _lsafExtDictionaryPath_ is blank. *;
%*                                                                                                      *;
%* @param  lsaf_location - Required - The case-insensitive output location for the exported external    *;
%*     dictionary file. Valid values: WORKPSACE and REPOSITORY.                                         *;
%* @param  lsaf_path - Required - The output path for the data set that contains the exported external  *;
%*     dictionaries. If the sas7bdat file extension is not specified, it is added during processing.    *;
%* @param  lsaf_overwrite - Optional - Indicates whether to overwrite an existing unversioned data      *;
%*     set. Valid values: 0 (Default): No, 1: Yes.                                                      *;
%* @param  lsaf_enableversioning - Optional - Indicates whether a new exported file is versioned.       *;
%*     This value is iggnored, if lsaf_location=WORKSPACE. Valid values: 0 (Default): No, 1: Yes        *;
%* @param  lsaf_versiontype - Conditional - The version type (MAJOR, MINOR, CUSTOM) to use to create    *;
%*     a file, if enableversioning is True. The default is MAJOR. This option is ignored, if            *;
%*     lsaf_location=WORKSPACE.                                                                         *;
%* @param  lsaf_customversion - Conditional - The version number to use to create a customized          *;
%*     versioned file. This option is ignored, if lsaf_location=WORKSPACE or if lsaf_versiontype is     *;
%*     not CUSTOM.                                                                                      *;
%* @param  lsaf_comment - Optional - The check-in comment to associate with the action of adding the    *;
%*      data set to the repository. This option is ignored, if lsaf_location=WORKSPACE.                 *;
%*                                                                                                      *;
%* @macrotype LSAF                                                                                      *;
%* @since 2.2                                                                                           *;
%* @exposure external                                                                                   *;
%*                                                                                                      *;
%* @history 2019-07-08  initial coding                                                                  *;
%*                                                                                                      *;

%macro lsaf_exportextdicts(
  lsaf_location=,
  lsaf_path=,
  lsaf_overwrite=,
  lsaf_enableversioning=,
  lsaf_versiontype=,
  lsaf_customversion=,
  lsaf_comment=
  ) /des='Exports external dictionaries to a specified location in the SAS Life Science Analytics Framework.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
          _lsafExtDictionaryPath_
  ;


  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           dictionaryPath  length=$2048
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300
    ;
    
    dictionaryPath="";
    
    %*****************************;
    %* Declare the java objects  *;
    %*****************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ExternalDictionaryService") ;
    declare javaobj results("com/sas/lsaf/macro/results/Result");
 
    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("exportExtDictionariesToDataSet", "%bquote(&lsaf_location)", "%bquote(&lsaf_path)", "%bquote(&lsaf_overwrite)", 
    "%bquote(&lsaf_enableversioning)", "%bquote(&lsaf_versiontype)", "%bquote(&lsaf_customversion)", "%bquote(&lsaf_comment)", results, dictionaryPath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not export the external dictionaries." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned the path for the created external dictionary file.";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (returnCode ne 0) 
      then put logMsgPrefix " The external dictionary was not exported.";
      else put logMsgPrefix " The external dictionary was exported to " dictionaryPath;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafExtDictionaryPath_=" dictionaryPath;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafExtDictionaryPath_", dictionaryPath);
  run;

%mend lsaf_exportextdicts;