%* lsaf_exportstandard                                                                                *;
%*                                                                                                    *;
%* Exports the metadata for a global standard. The metadata is stored in a SAS data set in the        *;
%* repository. The paths of the exported files are stored in a SAS data set.                          *;
%*                                                                                                    *;
%* Syntax:                                                                                            *;
%* %LSAF_EXPORTSTANDARD(LSAF_STANDARDID=lsaf-standardid,                                              *;
%*     LSAF_STATUS=lsaf-status,                                                                       *;
%*     LSAF_STANDARDVERSION=lsaf-standardversion,                                                     *;
%*     LSAF_PATH=lsaf-path,                                                                           *;
%*     LSAF_PREFIX=lsaf-prefix,                                                                       *;
%*     LSAF_OVERWRITE=lsaf-overwrite,                                                                 *;
%*     LSAF_ENABLEVERSIONING=lsaf-enableversioning,                                                   *;
%*     LSAF_VERSIONTYPEFORNEWFILES=lsaf-versiontypefornewfiles,                                       *;
%*     LSAF_CUSTOMVERSFORNEWFILES=lsaf-customversfornewfiles,                                         *;
%*     LSAF_VERSIONTYPEFOREXISTINGFILES=lsaf-versiontypeforexistingfiles,                             *;
%*     LSAF_CUSTOMVERSFOREXISTINGFILES=lsaf-customversforexistingfiles,                               *;
%*     LSAF_COMMENT=lsaf-comment,                                                                     *;
%*     SAS_RESULT_DSNAME=WORK.LSAFEXPORTSTANDARD);                                                    *;
%*                                                                                                    *;
%* @macvar _lsafRC_  The return code.                                                                 *;
%* @macvar _lsafMsg_ The return message.                                                              *;
%*                                                                                                    *;
%* @param lsaf_standardid - Required - The identifier of the data standard. The identifier can be     *;
%*     retrieved by calling the lsaf_getstandardid macro.                                             *;
%* @param lsaf_status - Required - The in-progress status of the standard. Valid values:              *;
%*     PUBLISHED, INPROGRESS, CHECKEDOUT. The default value is PUBLISHED.                             *;
%* @param  lsaf_standardversion - Optional - The version of a PUBLISHED standard to export. A blank   *;
%*     value returns the latest version. This option is ignored, if status is not PUBLISHED.          *;
%* @param  lsaf_path - Required - The output path for the exported files.                             *;
%* @param  lsaf_prefix - Required - The prefix of the SAS data sets that are created for the          *;
%*     exported data standards. All text is converted to lowercase.                                   *;
%* @param  lsaf_overwrite - Optional - Indicated whether existing nonversioned data sets are          *;
%*     overwritten. Valid values: 0 (Default): No, 1: Yes.                                            *;
%* @param  lsaf_enableversioning - Optional - Enable versioning for new exported files.               *;
%*     Valid values: 0 (Default): No, 1: Yes.                                                         *;
%* @param  lsaf_versiontypefornewfiles - Conditional - The version type (MAJOR, MINOR, CUSTOM) to     *;
%*     used to create a file, if enableversioning is True. Default: MAJOR.                            *;
%* @param  lsaf_customversfornewfiles - Conditional - The version number to use to create a           *;
%*     customized versioned file. This option is ignored, if versiontypefornewfiles is not CUSTOM.    *;
%* @param  lsaf_versiontypeforexistingfiles - Conditional - The version type (MAJOR, MINOR, CUSTOM)   *;
%*     for an existing file. Default: MINOR.                                                          *;
%* @param  lsaf_customversforexistingfiles - Conditional - The version number to use for an           *;
%*     existing customized versioned file. This option is ignored, if versiontypeforexistingfiles     *;
%*     is not CUSTOM.                                                                                 *;
%* @param  lsaf_comment - Optional - The check-in comment to associate with adding the data sets to   *;
%*     the repository.                                                                                *;
%* @param sas_result_dsname - Optional - The name of the output SAS data set to contain the path,     *;
%*            name, and location of the exported data standards SAS data sets, specified as           *;
%*            libref.dataset. The default is WORK.LSAFEXPORTSTANDARD. The data set contains a row for *;
%*            each export data set and columns with the following names. The data set is sorted by    *;
%*            path.                                                                                   *;
%*            - location: The location of the exported file (REPOSITORY).                             *;
%*            - name: The name of the exported file.                                                  *;
%*            - path: The path of the exported file.                                                  *;
%*                                                                                                    *;
%* @macrotype LSAF                                                                                    *;
%* @since 2.2                                                                                         *;
%* @exposure external                                                                                 *;
%*                                                                                                    *;
%* @history 2019-06-07  initial coding                                                                *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                     *;
%*                                                                                                    *;

%macro lsaf_exportstandard(
  lsaf_standardid=,
  lsaf_status=,
  lsaf_standardversion=,
  lsaf_path=,
  lsaf_prefix=,
  lsaf_overwrite=,
  lsaf_enableversioning=,
  lsaf_versiontypefornewfiles=,
  lsaf_customversfornewfiles=,
  lsaf_versiontypeforexistingfiles=,
  lsaf_customversforexistingfiles=,
  lsaf_comment=,
  sas_result_dsname=work.lsafexportstandard
  ) /des='Exports a global standard to a specified location in the SAS Life Science Analytics Framework.';

  %if "&sas_result_dsname" EQ "" %then %let sas_result_dsname=work.lsafexportstandard;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_result_dsname (label="Created by &SYSMACRONAME" keep=location path name sortPath);
    attrib returnCode         length=8
           message            length=$200
           logMsgPrefix       length=$70
           logMsgPrimary      length=$70
           logMsg             length=$300
           rowCount           length=8
           location           length=$10      label="Location"
           path               length=$2048    label="Export file Path"
           name               length=$256     label="Export file Name"
           sortpath           length=$2048    label="NEEDED FOR SORTING - will be dropped"
    ;

    %*****************************;
    %* Declare the java objects  *;
    %*****************************;
    declare javaobj srv("com/sas/lsaf/macro/service/StandardsService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasPathResult");
 
    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("exportStandardToDataSetsInRepository", "%bquote(&lsaf_standardid)", "%bquote(&lsaf_status)", "%bquote(&lsaf_standardversion)", 
      "%bquote(&lsaf_path)", "%bquote(&lsaf_prefix)", "%bquote(&lsaf_overwrite)", "%bquote(&lsaf_enableversioning)", "%bquote(&lsaf_versiontypefornewfiles)", 
      "%bquote(&lsaf_customversfornewfiles)", "%bquote(&lsaf_versiontypeforexistingfiles)", "%bquote(&lsaf_customversforexistingfiles)", 
      "%bquote(&lsaf_comment)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not export the global standard." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned the list of paths for the exported global standard.";
      results.callIntMethod("getSize", rowCount);
      results.callStringMethod("getLocation", location);
      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        sortPath = kstrip( kupcase( ktranslate(path,'1','_') ) );
        results.callStringMethod("getName", row, name);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount<=0) 
      then put logMsgPrefix " The global standard was not exported.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_result_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);
  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %if (&_lsafRowCount_ > 0) %then
  %do;
     proc sort data=&sas_result_dsname
               out=&sas_result_dsname (drop=sortpath);
         by sortpath;
     run;
  %end;
  %else
  %do;
     proc sql;
       alter table &sas_result_dsname
          drop sortpath;
     quit;
  %end;
  
  options &notesOption;

%mend lsaf_exportstandard;
