%* lsaf_exportstudydetails                                                                            *;
%*                                                                                                    *;
%* Exports the metadata for the study details. The metadata is stored in a SAS data set that is       *;
%* located in the workspace or the repository.                                                        *;
%*                                                                                                    *;
%* Syntax:                                                                                            *;
%* %LSAF_EXPORTSTUDYDETAILS(LSAF_STUDYID=lsaf-studyid,                                                *;
%*     LSAF_LOCATION=lsaf-location,                                                                   *;
%*     LSAF_PATH=lsaf-path,                                                                           *;
%*     LSAF_OVERWRITE=lsaf-overwrite,                                                                 *;
%*     LSAF_ENABLEVERSIONING=lsaf-enableversioning,                                                   *;
%*     LSAF_VERSIONTYPE=lsaf-versiontype,                                                             *;
%*     LSAF_CUSTOMVERSION=lsaf-customversion,                                                         *;
%*     LSAF_COMMENT=lsaf-comment);                                                                    *;
%*                                                                                                    *;
%* @macvar _lsafRC_  The return code.                                                                 *;
%* @macvar _lsafMsg_ The return message.                                                              *;
%* @macvar _lsafexportstudydetailspath_ The full path of the details data set, which includes         *;
%*     the file extension. If the data set is not created, _lsafexportstudydetailspath_ is blank.     *;
%*                                                                                                    *;
%* @param lsaf_studyid - Required - The identifier of the study. The identifier can be                *;
%*          retrieved by calling the lsaf_getstudyid macro.                                           *;
%* @param  lsaf_location - Required - The case-insensitive output location for the exported file.     *;
%*     Valid values: WORKSPACE and REPOSITORY.                                                        *;
%* @param  lsaf_path - Required - The case-sensitive output path for the exported files.              *;
%* @param  lsaf_overwrite - Optional - Indicates whether existing nonversioned data sets are          *;
%*     overwritten. Valid values: 0 (Default): No, 1: Yes.                                            *;
%* @param  lsaf_enableversioning - Optional - Indicates whether versioning is enabled for new         *;
%*     exported files. Valid values: 0 (Default): No, 1: Yes.                                         *;
%* @param  lsaf_versiontype - Conditional - The version type to use to create a file, if              *;
%*      enableversioning is True. Valid values: MAJOR (Default), MINOR, and CUSTOM. If                *;
%*      lsaf_location=WORKSPACE, this option is ignored.                                              *;
%* @param  lsaf_customversion - Conditional - The version number to use to create a customized        *;
%*     versioned file. If lsaf_location=WORKSPACE or if lsaf_versiontype is not CUSTOM, this option   *;
%*     is ignored.                                                                                    *;
%* @param  lsaf_comment - Optional - The check-in comment to associate with the action of adding the  *;
%*      data set to the repository. If lsaf_location=WORKSPACE, this option is ignored.               *;
%*                                                                                                    *;
%* @macrotype LSAF                                                                                    *;
%* @since 2.2                                                                                         *;
%* @exposure external                                                                                 *;
%*                                                                                                    *;
%* @history 2019-08-23  initial coding                                                                *;
%*                                                                                                    *;

%macro lsaf_exportstudydetails(
  lsaf_studyid=,
  lsaf_location=,
  lsaf_path=,
  lsaf_overwrite=,
  lsaf_enableversioning=,
  lsaf_versiontype=,
  lsaf_customversion=,
  lsaf_comment=
  ) /des='Exports study details to a specified location in the SAS Life Science Analytics Framework.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
		  _lsafexportstudydetailspath_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
	attrib returnCode         length=8
           message            length=$200
           logMsgPrefix       length=$70
           logMsgPrimary      length=$70
           logMsg             length=$300
		   exportPath         length=$2048
    ;

    %*****************************;
    %* Declare the java objects  *;
    %*****************************;
    declare javaobj srv("com/sas/lsaf/macro/service/StudyService") ;
    declare javaobj results("com/sas/lsaf/macro/results/Result");
 
    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("exportStudyDetails", "%bquote(&lsaf_studyid)",
      "%bquote(&lsaf_location)", "%bquote(&lsaf_path)", "%bquote(&lsaf_overwrite)", "%bquote(&lsaf_enableversioning)", "%bquote(&lsaf_versiontype)", 
      "%bquote(&lsaf_customversion)", "%bquote(&lsaf_comment)", results, exportPath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not export the study details." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned the path for the created details file.";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (returnCode ne 0) 
      then put logMsgPrefix " The study details were not exported.";
      else put logMsgPrefix " The study details were exported to " exportPath;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafexportstudydetailspath_=" exportPath;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafexportstudydetailspath_", kstrip(exportPath));
  run;

%mend lsaf_exportstudydetails;
