%* lsaf_getabsolutepath                                                                            *;
%*                                                                                                 *;
%* Gets an absolute path for an item relative to the specified base path. Neither path is          *;
%* validated for existence or for valid characters. Logging in to SAS Life Science Analytics       *;
%* Framework is not required to perform this operation.                                            *;
%*                                                                                                 *;
%* The macro variable _lsafAbsolutePath_ is set to the absolute path or blank, if the operation    *;
%* fails.                                                                                          *;
%*                                                                                                 *;
%* Syntax                                                                                          *;
%* %LSAF_GETABSOLUTEPATH(LSAF_BASE_PATH=lsaf-base-path, LSAF_RELATIVE_PATH=lsaf-relative-path);    *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%* @macvar _lsafAbsolutePath_ The absolute path.                                                   *;
%*                                                                                                 *;
%* @param lsaf_base_path - Required - The absolute path to the base container location.            *;
%* @param lsaf_relative_path - Required - The path of the file relative to the base path. A        *;
%*            value of . returns the full base path. A value of .. returns the parent of the base  *;
%*            path. For example, if the base path is /SAS/Folder1/Folder2/Folder3                  *;
%*            - A parameter value of  ./File4 returns an absolute path value of                    *;
%*              /SAS/Folder1/Folder2/Folder3/Folder4                                               *;
%*            - A parameter value of ../Folder4 returns an absolute path value of                  *;
%*              /SAS/Folder1/Folder2/Folder4                                                       *;
%*            - A parameter value of ../../Folder4 returns an absolute path value of               *;
%*              /SAS/Folder1/Folder4                                                               *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2017-01-19  original coding                                                            *;
%* @history 2019-02-13  update to 2.1                                                              *;
%*                                                                                                 *;

%macro lsaf_getabsolutepath(
  lsaf_base_path=,
  lsaf_relative_path=
  ) /des='Get the absolute path relative to the base path.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
          _lsafAbsolutePath_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300
           absolutePath    length=$2048
    ;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/UtilityService") ;
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("getAbsolutePath", "%bquote(&lsaf_base_path)", "%bquote(&lsaf_relative_path)", results, absolutePath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot generate absolute path for &lsaf_relative_path. based on &lsaf_base_path." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The absolute path was generated: " || kstrip(absolutePath) ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafAbsolutePath_= " absolutePath;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafAbsolutePath_", kstrip(absolutePath)) ;

  run;

%mend lsaf_getabsolutepath;
