%* lsaf_getallusers                                                                                *;
%*                                                                                                 *;
%* Gets the metadata for all of the user accounts that are defined. The metadata is stored in a    *;
%* SAS data set.                                                                                   *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETALLUSERS(<,SAS_DSNAME=SAS-data-set>);                                                  *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata      *;
%*            for the users, specified as libref.dataset.  The default value is                    *;
%*            WORK.LSAFGETALLUSERS.  The data set contains a row for each user, sorted by the user *;
%*            ID, and columns with the following names:                                            *;
%*            - userId: The unique login identifier for the account.                               *;
%*            - displayName: The display name for the account.                                     *;
%*            - firstName: The first name of the user.                                             *;
%*            - lastName: The last name of the user.                                               *;
%*            - email: The email address for the user.                                             *;
%*            - department: The business unit for the user.                                        *;
%*            - phone: The phone number for the user.                                              *;
%*            - isAccountActive: Indicates whether the account is active. Valid values:            *;
%*              0=no, 1=yes.                                                                       *;
%*            - accountLicenseType: The account license type.                                      *;
%*            - isAccountLocked: Indicates whether the account is locked. Valid values:            *;
%*              0=no, 1=yes.                                                                       *;
%*            - accountActivationDate: The date on which the account becomes active, represented   *;
%*              in Character format.                                                               *;
%*            - accountDeactivationDate: The date on which the account becomes inactive,           *;
%*              represented in Character format.                                                   *;
%*            - accountLockedBy: The user identifier of the user who locked the account.           *;
%*            - accountLockedDate: The date on which the account was locked, represented           *;
%*              in Character format.                                                               *;
%*            - accountType: The type of account. Valid values: INTERACTIVE and NON_INTERACTIVE.   *;
%*            - lastLogin: The date on which the account was last logged on to, represented in     *;
%*              Character format.                                                                  *;
%*            - loginAttempts: The current number of failed login attempts that are associated     *;
%*              with the account. If the number exceeds the configured threshold, this account is  *;
%*              locked. The counter is reset after a successful login.                             *;
%*            - passwordCreated: The date on which the current password was created, represented   *;
%*              in Character format.                                                               *;
%*            - passwordExpiration: The date on which the current password expires, represented    *;
%*              in Character format.                                                               *;
%*            - isPasswordExpired: Indicates whether the password has expired. Valid value:        *;
%*              0=no, 1=yes.                                                                       *;
%*            - dateAccountActivation: The date on which the account becomes active, represented   *;
%*              in SAS DateTime format.                                                            *;
%*            - dateAccountDeactivation: The date on which the account becomes inactive,           *;
%*              represented in SAS DateTime format.                                                *;
%*            - dateAccountLocked: The date on which the account was locked, represented in        *;
%*              SAS DateTime format.                                                               *;
%*            - dateLastLogin: The date on which the account was last logged on to, represented    *;
%*              in SAS DateTime format.                                                            *;
%*            - datePasswordCreated: The date on which the current password was created,           *;
%*              represented in SAS DateTime format.                                                *;
%*            - datePasswordExpiration: The date on which the current password expires,            *;
%*              represented in SAS DateTime format.                                                *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.3                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2013-10-30  initial coding                                                             *;
%* @history 2016-08-26  rebrand as LSAF                                                            *;
%* @history 2016-12-07  added account license type                                                 *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                  *;
%*                                                                                                 *;

%macro lsaf_getallusers(
  sas_dsname=work.lsafGetAllUsers
  ) /des='Get a list of all users defined in the system.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetAllUsers;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= userId displayName firstName lastName email department phone
                          accountLicenseType isAccountActive accountActivationDate accountDeactivationDate
                          isAccountLocked accountLockedBy accountLockedDate
                          accountType lastLogin loginAttempts
                          passwordCreated passwordExpiration isPasswordExpired
                          dateAccountActivation dateAccountDeactivation dateAccountLocked
                          dateLastLogin datePasswordCreated datePasswordExpiration
                          UPUSERID
  );

    attrib returnCode              length=8;
    attrib message                 length=$200;
    attrib logMsgPrefix            length=$70;
    attrib logMsgPrimary           length=$70;
    attrib logMsg                  length=$300;
    attrib rowCount                length=8;

    attrib userId                  length=$64    label="User ID";
    attrib UPUSERID                length=$64    label="NEEDED FOR SORTING - will be dropped";
    attrib displayName             length=$128   label="Display Name";
    attrib firstName               length=$64    label="First Name";
    attrib lastName                length=$64    label="Last Name";
    attrib email                   length=$128   label="Email Address";
    attrib department              length=$128   label="Department";
    attrib phone                   length=$64    label="Phone Number";

    attrib isAccountActive         length=8      label="Is Account Active";
    attrib accountLicenseType      length=$64    label="Account License Type";
    attrib accountActivationDate   length=$64    label="Account Activation Date";
    attrib accountDeactivationDate length=$64    label="Account Deactivation Date";

    attrib isAccountLocked         length=8      label="Is Account Locked";
    attrib accountLockedBy         length=$64    label="Account Locked By";
    attrib accountLockedDate       length=$64    label="Account Locked Date";

    attrib accountType             length=$32    label="Account Type";
    attrib lastLogin               length=$64    label="Last Login Date";
    attrib loginAttempts           length=8      label="Failed Login Attempts (Reset on Login)";

    attrib passwordCreated         length=$64    label="Password Creation Date";
    attrib passwordExpiration      length=$64    label="Password Expiration Date";
    attrib isPasswordExpired       length=8      label="Is Password Expired";

    label  dateAccountActivation="Account Activation Date (SAS DateTime Format)"
           dateAccountDeactivation="Account Deactivation Date (SAS DateTime Format)"
           dateAccountLocked="Account Locked Date (SAS DateTime Format)"
           dateLastLogin="Last Login Date (SAS DateTime Format)"
           datePasswordCreated="Password Creation Date (SAS DateTime Format)"
           datePasswordExpiration="Password Expiration Date (SAS DateTime Format)"
    ;
    format dateAccountActivation
           dateAccountDeactivation
           dateAccountLocked
           dateLastLogin
           datePasswordCreated
           datePasswordExpiration datetime.
    ;

    attrib accountActivationDateSasFormat        length=$64  label="Account Activation Date: SAS Format: USED FOR FORMATTING - will be dropped";
    attrib accountDeactivationDateSasFormat      length=$64  label="Account Deactivation Date: SAS Format: USED FOR FORMATTING - will be dropped";
    attrib accountLockedDateSasFormat            length=$64  label="Account Locked Date: SAS Format: USED FOR FORMATTING - will be dropped";
    attrib lastLoginSasFormat                    length=$64  label="Last Login Date: SAS Format: USED FOR FORMATTING - will be dropped";
    attrib passwordCreatedSasFormat              length=$64  label="Password Creation Date: SAS Format: USED FOR FORMATTING - will be dropped";
    attrib passwordExpirationSasFormat           length=$64  label="Password Expiration Date: SAS Format: USED FOR FORMATTING - will be dropped";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/UserService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasUsersResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getAllUsers", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve list of users.";
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" List of users was returned.";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getAccountLicenseTypeId", row, accountLicenseType);
        results.callStringMethod("getAccountActivationDate", row, accountActivationDate);
        results.callStringMethod("getAccountDeactivationDate", row, accountDeactivationDate);
        results.callStringMethod("getAccountLockedBy", row, accountLockedBy);
        results.callStringMethod("getAccountLockedDate", row, accountLockedDate);
        results.callStringMethod("getAccountType", row, accountType);
        results.callStringMethod("getDepartment", row, department);
        results.callStringMethod("getDisplayName", row, displayName);
        results.callStringMethod("getEmail", row, email);
        results.callStringMethod("getFirstName", row, firstName);
        results.callStringMethod("getLastLogin", row, lastLogin);
        results.callStringMethod("getLastName", row, lastName);
        results.callLongMethod("getLoginAttempts", row, loginAttempts);
        results.callStringMethod("getPasswordCreated", row, passwordCreated);
        results.callStringMethod("getPasswordExpiration", row, passwordExpiration);
        results.callStringMethod("getPhone", row, phone);
        results.callStringMethod("getUserId", row, userId);
        results.callIntMethod("isAccountActive", row, isAccountActive);
        results.callIntMethod("isAccountLocked", row, isAccountLocked);
        results.callIntMethod("isPasswordExpired", row, isPasswordExpired);

        results.callStringMethod("getAccountActivationDateSasFormat", row, accountActivationDateSasFormat);
        results.callStringMethod("getAccountDeactivationDateSasFormat", row, accountDeactivationDateSasFormat);
        results.callStringMethod("getAccountLockedDateSasFormat", row, accountLockedDateSasFormat);
        results.callStringMethod("getLastLoginSasFormat", row, lastLoginSasFormat);
        results.callStringMethod("getPasswordCreatedSasFormat", row, passwordCreatedSasFormat);
        results.callStringMethod("getPasswordExpirationSasFormat", row, passwordExpirationSasFormat);

        UPUSERID=upcase(userId);

        dateAccountActivation = input(accountActivationDateSasFormat, datetime.);
        dateAccountDeactivation = input(accountDeactivationDateSasFormat, datetime.);
        dateAccountLocked = input(accountLockedDateSasFormat, datetime.);
        dateLastLogin = input(lastLoginSasFormat, datetime.);
        datePasswordCreated = input(passwordCreatedSasFormat, datetime.);
        datePasswordExpiration = input(passwordExpirationSasFormat, datetime.);

        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No users were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;
  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    proc sort data=&sas_dsname;
      by upuserid;
    run;
  %end;
  %***********************************;
  %* Cleanup                         *;
  %***********************************;
  proc sql;
     alter table &sas_dsname
        drop UPUSERID;
  quit;

  options &notesOption;

%mend lsaf_getallusers;
