%* lsaf_getassignedmembers                                                                       *;
%*                                                                                               *;
%* Gets the metadata for all of the members that are assigned to the specified context. The      *;
%* metadata is stored in a SAS data set.                                                         *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETASSIGNEDMEMBERS(LSAF_PATH=lsaf-path <, SAS_DSNAME=SAS-data-set>);                    *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - Required - The path of the context that contains the members.              *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata    *;
%*            for the members, specified as libref.dataset. The default value is                 *;
%*            WORK.LSAFGETASSIGNEDMEMBERS. The data set contains a row for each member and       *;
%*            columns with the following names. The data set is sorted by the member type        *;
%*            and principalId.                                                                   *;
%*            - contextPath: The path of the context in which the members are assigned.          *;
%*            - type:  The member type. Valid values: USER and GROUP.                            *;
%*            - principalId: The name of the member. The returned values are either a group      *;
%*              name or a user ID.                                                               *;
%*            - description: The description of the member. The returned values are either a     *;
%*              group name or a user display name.                                               *;
%*            - grpSrcCtxt: The context path in which the group member is defined, if the member *;
%*              is a group.                                                                      *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                *;
%*                                                                                               *;

%macro lsaf_getassignedmembers(
  lsaf_path=,
  sas_dsname=work.lsafGetAssignedMembers
  ) /des='Get a list of the assigned members in an SAS Life Science Analytics Framework context.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetAssignedMembers;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data &sas_dsname (
          label="Created by &SYSMACRONAME"
          keep=contextPath type principalId description grpSrcCtxt
          );
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib contextPath     length=$2048     label="Context Path" ;
    attrib type            length=$32       label="Principal Type";
    attrib principalId     length=$256      label="Principal Name";
    attrib description     length=$128      label="Principal Description";
    attrib grpSrcCtxt      length=$2048     label="Group Context Source Path";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/MembershipService");
    declare javaobj results("com/sas/lsaf/macro/results/SasPrincipalsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getAssignedMembers", "%bquote(&lsaf_path)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve assigned members for &lsaf_path" ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Assigned members were returned for &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        contextPath="&lsaf_path";
        results.callStringMethod("getType", row, type);
        results.callStringMethod("getName", row, principalId);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getContextPath", row, grpSrcCtxt);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No members were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
   %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by type principalId;
    run;
    options &notesOption;
  %end;

%mend lsaf_getassignedmembers;
