%* lsaf_getjobinputs                                                                              *;
%*                                                                                                *;
%* Gets the metadata for the inputs for the specified job in the repository. The metadata is      *;
%* stored in a SAS data set. The list does not include the input programs.                        *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETJOBINPUTS(LSAF_PATH=lsaf-path <LSAF_VERSION=lsaf-version,                             *;
%*                  SAS_DSNAME=SAS-data-set>);                                                    *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - Required - The path of the job.                                             *;
%* @param lsaf_version - Optional - The version of the job. If not specified, the latest version  *;
%*            is used.                                                                            *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata     *;
%*            for the inputs, specified as libref.dataset. The default value is                   *;
%*            WORK.LSAFGETJOBINPUTS. The data set contains a row for each job input and columns   *;
%*            with the following names. The data set retains the order of the inputs.             *;
%*            - jobPath: The path of the job.                                                     *;
%*            - jobVersion: The job version.                                                      *;
%*            - inputPath: The path of the input file.                                            *;
%*            - inputType: The item type for the input. Valid values: CONTAINER and FILE.         *;
%*            - inputVersion: The version of the input file. A blank value specifies the latest   *;
%*              version of the input file is used by the job.                                     *;
%*            - inputIncludeSubFolders: Indicates whether the subfolders of a container input     *;
%*              are included during job execution.                                                *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.4                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2014-04-22  initial coding.                                                           *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%* @history 2019-02-11  update to 2.1                                                             *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                 *;
%*                                                                                                *;

%macro lsaf_getjobinputs(
   lsaf_path=,
   lsaf_version=,
   sas_dsname=work.lsafGetJobInputs
   ) /des='Get a list of the inputs for a job.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetJobInputs;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %**********************************************;
   %* Start the datastep to call the java code   *;
   %**********************************************;
   data &sas_dsname (label="Created by &SYSMACRONAME"
                     keep=jobPath jobVersion inputType inputPath inputVersion inputIncludeSubFolders
   );
   attrib returnCode      length=8
          message         length=$200
          logMsgPrefix    length=$70
          logMsgPrimary   length=$70
          logMsg          length=$300
          rowCount        length=8
          
          jobPath         length=$2048      label="Job Path"
          jobVersion      length=$32        label="Job Version"
          inputPath       length=$2048      label="Input Path"
          inputType       length=$32        label="Input Type"
          inputVersion    length=$32        label="Input File Version"

          inputIncludeSubFolders  length=8  label="Include Input Container SubFolders"
          includeSubFolders       length=$5 label="FOR INTERNAL PROCESSING ONLY.  WILL BE DROPPED."
   ;

   rowCount=0;

   %***********************************;
   %* Declare the java objects        *;
   %***********************************;
   declare javaobj srv("com/sas/lsaf/macro/service/JobService") ;
   declare javaobj results("com/sas/lsaf/macro/results/SasJobItemSpecificationResult");

   %***********************************;
   %* Attempt the operation           *;
   %***********************************;
   srv.callStaticVoidMethod("getRepositoryJobInputs", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", results) ;

   %***********************************;
   %* Retrieve the results            *;
   %***********************************;
   results.callIntMethod("getReturnCode", returnCode);
   results.callStringMethod("getReturnMessage", message);

   %***********************************;
   %* Process the results             *;
   %***********************************;
   logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
   if (returnCode ne 0) then
   do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      if "&lsaf_version" NE ""
        then logMsg=" Cannot retrieve inputs for job &lsaf_path, version &lsaf_version.";
        else logMsg=" Cannot retrieve inputs for job &lsaf_path.";
   end;
   else
   do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Inputs were returned for job &lsaf_path" ;
      results.callStringMethod("getJobPath", jobPath);
      results.callStringMethod("getJobVersion", jobVersion);
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
         results.callStringMethod("getPath", row, inputPath);
         results.callStringMethod("getType", row, inputType);
         results.callStringMethod("getFileVersion", row, inputVersion);
         results.callStringMethod("getIncludeSubFolders", row, includeSubFolders);
         inputIncludeSubFolders=input(includeSubFolders,best.);
         output;
      end;
   end;

   %***********************************;
   %* Print messages in log           *;
   %***********************************;
   put;
   put logMsgPrimary " " logMsg;
   put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
   if (rowCount=0) 
     then put logMsgPrefix " No inputs were found.";
     else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
   put logMsgPrefix " _lsafMsg_= " message ;
   put logMsgPrefix " _lsafRC_= " returnCode ;
   put;

   %***********************************;
   %* Set the macro variables         *;
   %***********************************;
   call symputx("_lsafRC_", returnCode);
   call symputx("_lsafMsg_", message);

   run;

%mend lsaf_getjobinputs;
