%* lsaf_getjobmanifestinfo                                                                        *;
%*                                                                                                *;
%* Gets the information for a job manifest in the repository. The information is stored in a SAS  *;
%* data set. The output data set can be used as an input data set for the lsaf_createworkspacejob *;
%* macro.                                                                                         *;
%*                                                                                                *;
%* This is the information that is displayed on the Details tab of the job manifest and the       *;
%* information that concerns the versioning of new output files.                                  *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETJOBMANIFESTINFO(LSAF_PATH=lsaf-path <, LSAF_VERSION=lsaf-version,                     *;
%*                          SAS_DSNAME=SAS-data-set>);                                            *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - Required - The case-sensitive path of the job manifest.                     *;
%* @param lsaf_version - Optional - The specific version number of the job manifest. If not       *;
%*            specified, the information for the latest version is returned. Specifying a version *;
%*            for a non-versioned job manifest causes an error.                                   *;
%* @param sas_dsname - Optional - The name of the output SAS data set that contains the metadata  *;
%*            for the manifest, specified as libref.dataset. The default value is                 *;
%*            WORK.LSAFGETJOBMANIFESTINFO. The data set contains variables that represent each    *;
%*            element of the metadata for the job manifest. The data set is sorted by the NAME    *;
%*            variable. The following metadata is represented in the data set.                    *;
%*            - path: The path of the job manifest.                                               *;
%*            - version: The version number that is associated with the job manifest.             *;
%*            - name: The name of the property or attribute.                                      *;
%*            - value: The value of the property or attribute.                                    *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 2.3                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2019-12-03  initial coding.                                                           *;
%*                                                                                                *;

%macro lsaf_getjobmanifestinfo(
  lsaf_path=,
  lsaf_version=,
  sas_dsname=work.lsafGetJobManifestInfo
  ) /des='Get the metadata for SAS Life Science Analytics Framework repository job manifest.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetJobManifestInfo;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_
         notesOption
  ;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=path version name value);
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300

           path            length=$2048    label="Job Manifest Path"
           version         length=$32      label="Job Manifest Version Number"
           name            length=$256     label="Property Name"
           value           length=$2048    label="Property Value"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/JobSubmissionService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasManifestPropertiesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getJobManifestInfo", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not retrieve job manifest information." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Retrieved job manifest information.";
      results.callStringMethod("getManifestPath", path);
      results.callStringMethod("getManifestVersion", version);
      results.callIntMethod("getSize", rowcount);
      do row=0 to rowCount - 1;
         results.callStringMethod("getPropertyName", row, name);
         results.callStringMethod("getPropertyValue", row, value);
         output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " The job manifest information was not found.";
      else put logMsgPrefix  rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by name;
    run;
    options &notesOption;
  %end;

%mend lsaf_getjobmanifestinfo;
