%* lsaf_getjobmanifestmetrics                                                                      *;
%*                                                                                                 *;
%* Gets the metrics for the specified job manifest in the repository. The data is stored in a      *;
%* SAS data set.                                                                                   *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETJOBMANIFESTMETRICSS(LSAF_PATH=lsaf-path <,LSAF_VERSION=lsaf-version,                   *;
%*                  SAS_DSNAME=SAS-data-set>);                                                     *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - Required - The path of the job manifest.                                     *;
%* @param lsaf_version - Optional - The version of the job manifest. If not specified, the latest  *;
%*             version is used.                                                                    *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the manifest      *;
%*            metrics, specified as libref.dataset. The default value is                           *;
%*            WORK.LSAFGETJOBMANIFESTMETRICS. The data set contains a row for each job manifest    *;
%*            type and columns with the following names. The data set retains the order of the     *;
%*            manifest types.                                                                      *;
%*            - jobManifestPath: The path of the job manifest.                                     *;
%*            - jobManifestVersion: The job manifest version.                                      *;
%*            - type: The type of metric.                                                          *;
%*            - total: The total for the metric.                                                   *;
%*            - size: The size of the metric.                                                      *;
%*            - formattedsize: The formatted size (in bytes, KB, MB, or TB).                       *;
%*            - time: The time or interval of the metric.                                          *;
%*            - formattedtime: The formatted time or interval of the metric, which is displayed    *;
%*              as h:m:s:ms.                                                                       *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 2.3                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2019-11-08  initial coding.                                                            *;
%*                                                                                                 *;

%macro lsaf_getjobmanifestmetrics(
   lsaf_path=,
   lsaf_version=,
   sas_dsname=work.lsafgetjobmanifestmetrics
   ) /des='Get the metrics for a job manifest.';

   %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafgetjobmanifestmetrics;

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %**********************************************;
   %* Start the datastep to call the java code   *;
   %**********************************************;
   data &sas_dsname (label="Created by &SYSMACRONAME"
                     keep=jobManifestPath jobManifestVersion type total size formattedsize time formattedtime
   );
   attrib returnCode      length=8
          message         length=$200
          logMsgPrefix    length=$70
          logMsgPrimary   length=$70
          logMsg          length=$300
          rowCount        length=8

          jobManifestPath     length=$2048     label="Job Manifest Path"
          jobManifestVersion  length=$32       label="Job Manifest Version"
          type                length=$2048     label="Type"
          total               length=8         label="Total"
          size                length=8         label="Size (Bytes)"
          formattedsize       length=$20       label="Size (Formatted)"
          time                length=8         label="Time (Microseconds)"
          formattedtime       length=$20       label="Time (h:m:s:ms)"
   ;

   rowCount=0;

   %***********************************;
   %* Declare the java objects        *;
   %***********************************;
   declare javaobj srv("com/sas/lsaf/macro/service/JobSubmissionService") ;
   declare javaobj results("com/sas/lsaf/macro/results/SasManifestMetricsResult");

   %***********************************;
   %* Attempt the operation           *;
   %***********************************;
   srv.callStaticVoidMethod("getJobManifestMetricsByPath", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", results) ;

   %***********************************;
   %* Retrieve the results            *;
   %***********************************;
   results.callIntMethod("getReturnCode", returnCode);
   results.callStringMethod("getReturnMessage", message);

   %***********************************;
   %* Process the results             *;
   %***********************************;
   logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
   if (returnCode ne 0) then
   do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      if "&lsaf_version" NE ""
        then logMsg=" Cannot retrieve metrics for job manifest &lsaf_path, version &lsaf_version" ;
        else logMsg=" Cannot retrieve metrics for job manifest &lsaf_path" ;
   end;
   else
   do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Metrics were returned for job manifest &lsaf_path" ;
      results.callStringMethod("getManifestPath", jobManifestPath);
      results.callStringMethod("getManifestVersion", jobManifestVersion);
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
         results.callStringMethod("getMetricType", row, type);
         results.callIntMethod("getMetricTotal", row, total);
         if total lt 0 then total=.;
         results.callLongMethod("getMetricSize", row, size);
         results.callStringMethod("getMetricFormattedSize", row, formattedsize);
         if size lt 0 then size=.;
         results.callLongMethod("getMetricTime", row, time);
         if time lt 0 then time=.;
         results.callStringMethod("getMetricFormattedTime", row, formattedtime);
         output;
      end;
   end;

   %***********************************;
   %* Print messages in log           *;
   %***********************************;
   put;
   put logMsgPrimary " " logMsg;
   put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
   if (rowCount=0) 
     then put logMsgPrefix " No metrics were found.";
     else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
   put logMsgPrefix " _lsafMsg_= " message ;
   put logMsgPrefix " _lsafRC_= " returnCode ;
   put;

   %***********************************;
   %* Set the macro variables         *;
   %***********************************;
   call symputx("_lsafRC_", returnCode);
   call symputx("_lsafMsg_", message);

   run;

%mend lsaf_getjobmanifestmetrics;