%* lsaf_getjobmanifestparameters                                                                   *;
%*                                                                                                 *;
%* Gets the metadata for the parameters for the specified job manifest in the repository. The      *;
%* metadata is stored in a SAS data set.                                                           *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETJOBMANIFESTPARAMETERS(LSAF_PATH=lsaf-path <,LSAF_VERSION=lsaf-version,                 *;
%*                  SAS_DSNAME=SAS-data-set>);                                                     *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - Required - The case-sensitive path of the job manifest.                      *;
%* @param lsaf_version - Optional - The version of the job manifest. If not specified, the latest  *;
%*          version is used.                                                                       *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata      *;
%*          for the parameters, specified as libref.dataset. The default value is                  *;
%*          WORK.LSAFGETJOBMANIFESTPARAMETERS. The data set contains a row for each job manifest   *;
%*          parameter and columns with the following names.                                        *;
%*            - jobManifestPath: The path of the job manifest.                                     *;
%*            - jobManifestVersion: The job manifest version.                                      *;
%*            - jobLocation: The job source location.                                              *;
%*            - name: The label of the parameter.                                                  *;
%*            - type: The type of the parameter.                                                   *;
%*            - label: The label of the parameter.                                                 *;
%*            - value: The value of the parameter.                                                 *;
%*            - version: The version of the file that is used in the job submission, or blank, if  *;
%*              file is not versioned.                                                             *;
%*            - includeSubfolders: Indicates whether to include subfolders. Valid values: 0: No,   *;
%*              1: Yes.                                                                            *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 2.3                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2019-11-13  initial coding.                                                            *;
%*                                                                                                 *;

%macro lsaf_getjobmanifestparameters(
   lsaf_path=,
   lsaf_version=,
   sas_dsname=work.lsafGetJobManifestParameters
   ) /des='Get a list of the parameters for a job manifest.';

   %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetJobManifestParameters;

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
  %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %**********************************************;
   %* Start the datastep to call the java code   *;
   %**********************************************;
   data &sas_dsname (label="Created by &SYSMACRONAME"
                     keep=jobManifestPath jobManifestVersion jobLocation name type label value version includeSubfolders
   );
   attrib returnCode      length=8
          message         length=$200
          logMsgPrefix    length=$70
          logMsgPrimary   length=$70
          logMsg          length=$300
          rowCount        length=8

          jobManifestPath    length=$2048     label="Job Manifest Path"
          jobManifestVersion length=$32       label="Job Manifest Version"
          jobLocation        length=$20       label="Job Source Location"
          name               length=$32       label="Parameter Name"
          type               length=$32       label="Parameter Type"
          label              length=$60       label="Parameter Label"
          value              length=$2048     label="Parameter Value"
          version            length=$10       label="Parameter File Version"
          includeSubfolders  length=$5        label="Include Subfolders"
   ;

   rowCount=0;

   %***********************************;
   %* Declare the java objects        *;
   %***********************************;
   declare javaobj srv("com/sas/lsaf/macro/service/JobSubmissionService") ;
   declare javaobj results("com/sas/lsaf/macro/results/SasManifestParametersResult");

   %***********************************;
   %* Attempt the operation           *;
   %***********************************;
   srv.callStaticVoidMethod("getJobManifestParameters", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", results) ;

   %***********************************;
   %* Retrieve the results            *;
   %***********************************;
   results.callIntMethod("getReturnCode", returnCode);
   results.callStringMethod("getReturnMessage", message);

   %***********************************;
   %* Process the results             *;
   %***********************************;
   logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
   if (returnCode ne 0) then
   do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      if "&lsaf_version" NE ""
        then logMsg=" Cannot retrieve parameters for job manifest &lsaf_path, version &lsaf_version" ;
        else logMsg=" Cannot retrieve parameters for job manifest &lsaf_path" ;
   end;
   else
   do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Parameters were returned for job manifest &lsaf_path" ;
      results.callStringMethod("getManifestPath", jobManifestPath);
      results.callStringMethod("getManifestVersion", jobManifestVersion);
      results.callIntMethod("getSize", rowCount);
      results.callStringMethod("getJobSourceLocation", jobLocation);
      do row=0 to rowCount - 1;
         results.callStringMethod("getName", row, name);
         results.callStringMethod("getType", row, type);
         results.callStringMethod("getLabel", row, label);
         results.callStringMethod("getParameterValue", row, value);
         results.callStringMethod("getFileVersion", row, version);
         results.callStringMethod("getContainerIncludeSubfolders", row, includeSubfolders);
         output;
      end;
   end;

   %***********************************;
   %* Print messages in log           *;
   %***********************************;
   put;
   put logMsgPrimary " " logMsg;
   put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
   if (rowCount=0) 
     then put logMsgPrefix " No parameters were found.";
     else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
   put logMsgPrefix " _lsafMsg_= " message ;
   put logMsgPrefix " _lsafRC_= " returnCode ;
   put;

   %***********************************;
   %* Set the macro variables         *;
   %***********************************;
   call symputx("_lsafRC_", returnCode);
   call symputx("_lsafMsg_", message);

   run;

%mend lsaf_getjobmanifestparameters;