%* lsaf_getjoboutputs                                                                              *;
%*                                                                                                 *;
%* Gets the metadata for the outputs for the specified job in the repository. The metadata is      *;
%* stored in a SAS data set.                                                                       *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETJOBOUTPUTS(LSAF_PATH=lsaf-path <LSAF_VERSION=lsaf-version,                             *;
%*                  SAS_DSNAME=SAS-data-set>);                                                     *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - Required - The path of the job.                                              *;
%* @param lsaf_version - Optional - The version of the job. If not specified, the latest version   *;
%*            is used.                                                                             *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata      *;
%*            for the outputs, specified as libref.dataset. The default value is                   *;
%*            WORK.LSAFGETJOBOUTPUTS. The data set contains a row for each job output and columns  *;
%*            with the following names. The data set retains the order of the outputs.             *;
%*            - jobPath: The path of the job.                                                      *;
%*            - jobVersion: The job version.                                                       *;
%*            - outputPath: The path of the output location.                                       *;
%*            - outputIncludeSubFolders: Indicates whether the subfolders of an output container   *;
%*              are included during job execution.                                                 *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.4                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2014-04-22  initial coding.                                                            *;
%* @history 2016-08-26  rebrand as LSAF                                                            *;
%* @history 2019-02-11  update to 2.1                                                              *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                  *;
%*                                                                                                 *;

%macro lsaf_getjoboutputs(
   lsaf_path=,
   lsaf_version=,
   sas_dsname=work.lsafGetJobOutputs
   ) /des='Get a list of the outputs for a job.';
  
   %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetJobOutputs;

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %**********************************************;
   %* Start the datastep to call the java code   *;
   %**********************************************;
   data &sas_dsname (label="Created by &SYSMACRONAME"
                     keep=jobPath jobVersion outputPath outputIncludeSubFolders
   );

   attrib returnCode      length=8
          message         length=$200
          logMsgPrefix    length=$70
          logMsgPrimary   length=$70
          logMsg          length=$300
          rowCount        length=8

          jobPath         length=$2048       label="Job Path"
          jobVersion      length=$32         label="Job Version"
          outputPath      length=$2048       label="Output Path"
          
          outputIncludeSubFolders  length=8  label="Include Output Container SubFolders"
          includeSubFolders        length=$5 label="FOR INTERNAL PROCESSING ONLY.  WILL BE DROPPED."
   ;

   rowCount=0;

   %***********************************;
   %* Declare the java objects        *;
   %***********************************;
   declare javaobj srv("com/sas/lsaf/macro/service/JobService") ;
   declare javaobj results("com/sas/lsaf/macro/results/SasJobItemSpecificationResult");

   %***********************************;
   %* Attempt the operation           *;
   %***********************************;
   srv.callStaticVoidMethod("getRepositoryJobOutputs", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", results) ;

   %***********************************;
   %* Retrieve the results            *;
   %***********************************;
   results.callIntMethod("getReturnCode", returnCode);
   results.callStringMethod("getReturnMessage", message);

   %***********************************;
   %* Process the results             *;
   %***********************************;
   logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
   if (returnCode ne 0) then
   do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      if "&lsaf_version" NE ""
        then logMsg=" Cannot retrieve outputs for job &lsaf_path, version &lsaf_version." ;
        else logMsg=" Cannot retrieve outputs for job &lsaf_path" ;
   end;
   else
   do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Outputs were returned for job &lsaf_path" ;
      results.callStringMethod("getJobPath", jobPath);
      results.callStringMethod("getJobVersion", jobVersion);
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
         results.callStringMethod("getPath", row, outputPath);
         results.callStringMethod("getIncludeSubFolders", row, includeSubFolders);
         outputIncludeSubFolders=input(includeSubFolders,best.);
         output;
      end;
   end;

   %***********************************;
   %* Print messages in log           *;
   %***********************************;
   put;
   put logMsgPrimary " " logMsg;
   put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
   if (rowCount=0) 
     then put logMsgPrefix " No outputs were found.";
     else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
   put logMsgPrefix " _lsafMsg_= " message ;
   put logMsgPrefix " _lsafRC_= " returnCode ;
   put;

   %***********************************;
   %* Set the macro variables         *;
   %***********************************;
   call symputx("_lsafRC_", returnCode);
   call symputx("_lsafMsg_", message);

   run;

%mend lsaf_getjoboutputs;
