%* lsaf_getpfsetupelements                                                                         *;
%*                                                                                                 *;
%* Gets the metadata for the flow setup elements for the specified process flow within the         *;
%* specified context. The metadata is stored in a SAS data set.                                    *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETPFSETUPELEMENTS(LSAF_PATH=lsaf-path, LSAF_processflow=process-flow-name                *;
%*              <,SAS_DSNAME=SAS-data-set>);                                                       *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - Required - The path of the context in which the process flow is located.     *;
%* @param lsaf_processflow - Required - The name of the process flow.                              *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the elements      *;
%*            for the process flow data, specified as libref.dataset. The default value is         *;
%*            WORK.LSAFGETPFSETUPELEMENTS. The data set contains a row for each element and        *;
%*            columns with the following names. The data set is not sorted.                        *;
%*            The following metadata will be represented in the data set.                          *;
%*            - processFlowPath: The path of the context in which the process flow is located.     *;
%*            - processFlowName: The name of the process flow.                                     *;
%*            - elementType: The type of the element.                                              *;
%*            - elementID: The unique ID  of the element.                                          *;
%*            - elementName: The name of the element.                                              *;
%*            - elementDescription: The description of the element.                                *;
%*                                                                                                 *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-12-07  original creation.                                                         *;
%* @history 2019-02-12  update to 2.1                                                              *;
%* @history 2019-03-15  rename from lsaf_getprocessflowelements to lsaf_getpfsetupelements         *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                  *;
%*                                                                                                 *;

%macro lsaf_getpfsetupelements(
  lsaf_path=,
  lsaf_processflow=,
  sas_dsname=work.lsafgetpfsetupelements
  ) /des='Get a list of process flow setup elements of a specific process flow in a SAS Life Science Analytics Framework context.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafgetpfsetupelements;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);


  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= processFlowPath processFlowName
                          elementID elementName elementDescription elementType);

    attrib returnCode                length=8
           message                   length=$200
           logMsgPrefix              length=$70
           logMsgPrimary             length=$70
           logMsg                    length=$300
           rowCount                  length=8

           processFlowPath           length=$2048   label="Context Path"
           processFlowName           length=$256    label="Process Flow Name"

           elementType               length=$64     label="Element Type"
           elementID                 length=$256    label="Element Name"
           elementName               length=$256    label="Element Display Name"
           elementDescription        length=$2048   label="Element Value"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ProcessFlowSetupService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasProcessFlowElementsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getAllSetupElements", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process flow elements for &lsaf_processflow in &lsaf_path" ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Process flow elements were returned for &lsaf_processflow in &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getProcessFlowContext", processFlowPath);
        results.callStringMethod("getProcessFlowName", processFlowName);
        results.callStringMethod("getId", row, elementID);
        results.callStringMethod("getName", row, elementName);
        results.callStringMethod("getDescription", row, elementDescription);
        results.callStringMethod("getType", row, elementType);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No process flow elements were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_getpfsetupelements;
