%* lsaf_getpfsetupjobparameters                                                                     *;
%*                                                                                                  *;
%* Gets the override parameters for the job setup elements for the process flow within the          *;
%* specified context. The parameters are stored in a SAS data set.                                  *;
%*                                                                                                  *;
%* Job setup elements maintain a list of the parameters that are used to override the default       *;
%* parameters when the job task executes. The output data set from the lsaf_getjobparameters        *;
%* macro can be used to get the full list and metadata for the job parameters.                      *;
%*                                                                                                  *;
%* Syntax:                                                                                          *;
%* %LSAF_GETPFSETUPJOBPARAMETERS(LSAF_PATH=lsaf-path, LSAF_PROCESSFLOW=process-flow-name            *;
%*     <, SAS_DSNAME=SAS-data-set>);                                                                *;
%*                                                                                                  *;
%* @macvar _lsafRC_  The return code.                                                               *;
%* @macvar _lsafMsg_ The return message.                                                            *;
%*                                                                                                  *;
%* @param lsaf_path - Required - The path of the context in which the process flow is located.      *;
%* @param lsaf_processflow - Required - The name of the process flow.                               *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the override       *;
%*            parameters for the process flow element, specified as libref.dataset.                 *;
%*            The default value is WORK.LSAFGETPFSETUPJOBPARAMETERS. The data set contains a        *;
%*            row for each override parameter and columns with the following names. The data set    *;
%*            is sorted by the element ID and parameter name.                                       *;
%*            The following metadata is represented in the data set.                                *;
%*            - processFlowPath: The path of the context in which the process flow is located.      *;
%*            - processFlowName: The name of the process flow.                                      *;
%*            - elementID: The ID for the element applicable for the parameter.                     *;
%*            - jobPath: The path for the job.                                                      *;
%*            - jobVersion: The version of the job.                                                 *;
%*            - name: The name of the parameter.                                                    *;
%*            - type: The type of the parameter. Valid values:CHARACTER, CHARACTER_MASKED, NUMERIC, *;
%*              DATE, FILE, or FOLDER.                                                              *;
%*            - value: The value of the parameter.                                                  *;
%*            - fileVersion: The file version, if the parameter is of type FILE. A blank value      *;
%*              specifies that either the file is not versioned or the file is versioned but no     *;
%*              version was specified.                                                              *;
%*            - includeSubFolders: Indicates whether the subfolders of a container are included     *;
%*              during job execution.                                                               *;
%*                                                                                                  *;
%* @macrotype LSAF                                                                                  *;
%* @since 1.5                                                                                       *;
%* @exposure external                                                                               *;
%*                                                                                                  *;
%* @history 2016-12-14  original creation.                                                          *;
%* @history 2019-02-12  update to 2.1                                                               *;
%* @history 2019-03-15  rename from lsaf_getpfjobparameters to lsaf_getpfsetupjobparameters         *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                   *;
%*                                                                                                  *;

%macro lsaf_getpfsetupjobparameters(
  lsaf_path=,
  lsaf_processflow=,
  sas_dsname=work.lsafGetPfSetupJobParameters
  ) /des='Get a list of process flow job setup element override parameters for the specific process flow.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetPfSetupJobParameters;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;
  %let notesOption=%sysfunc(getoption(notes));
  options notes;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= processFlowPath processFlowName elementID
                          jobPath jobVersion name type value fileVersion includeSubFolders);

    attrib returnCode        length=8
           message           length=$200
           logMsgPrefix      length=$70
           logMsgPrimary     length=$70
           logMsg            length=$300
           rowCount          length=8
           processFlowPath   length=$2048  label="Context Path"
           processFlowName   length=$256   label="Process Flow Name"
           elementID         length=$256   label="Element ID"
           jobPath           length=$2048  label="Job Path"
           jobVersion        length=$32    label="Job Version"
           name              length=$256   label="Parameter Name"
           type              length=$32    label="Parameter Type"
           value             length=$2048  label="Override Value"
           fileVersion       length=$32    label="File Version if File Type"
           includeSubFolders length=8      label="Include Container SubFolders"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ProcessFlowSetupService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasProcessFlowElementsParametersResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getJobElementsParameters", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process flow job element override parameters for &lsaf_processflow in &lsaf_path" ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Process flow job element override parameters were returned for &lsaf_processflow in &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getProcessFlowContext", processFlowPath);
        results.callStringMethod("getProcessFlowName", processFlowName);
        results.callStringMethod("getElementID", row, elementID);
        results.callStringMethod("getJobPath", row, jobPath);
        results.callStringMethod("getJobVersion", row, jobVersion);
        results.callStringMethod("getParameterName", row, name);
        results.callStringMethod("getParameterType", row, type);
        results.callStringMethod("getParameterValue", row, value);
        results.callStringMethod("getParameterFileVersion", row, fileVersion);
        results.callIntMethod("getParameterIncludeSubfolders", row, includeSubFolders);       
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No process flow job elements override parameters were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    options nonotes;
    proc sort data=&sas_dsname;
      by elementID name;
    run;
  %end;
  
  options &notesOption;
  
%mend lsaf_getpfsetupjobparameters;
