%* lsaf_getpfsetupnotifrecips                                                                      *;
%*                                                                                                 *;
%* Gets the recipients for notification setup elements for the process flow within the specified   *;
%* context. The recipients are stored in a SAS data set.                                           *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETPFSETUPNOTIFRECIPS(LSAF_PATH=lsaf-path,                                                *;
%*      LSAF_PROCESSFLOW=process-flow-name <, SAS_DSNAME=SAS-data-set>);                           *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - Required - The path of the context in which the process flow is located.     *;
%* @param lsaf_processflow - Required - The name of the process flow.                              *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata      *;
%*            for the recipients, specified as libref.dataset. The default value is                *;
%*            WORK.LSAFPFSETUPGETNOTIFRECIPS. The data set contains a row for each recipient       *;
%*            and columns with the following names. The data set is sorted by the element id,      *;
%*            recipient type, and principalId.                                                     *;
%*            - processFlowPath: The path of the context in which the process flow is located.     *;
%*            - processFlowName: The name of the process flow.                                     *;
%*            - elementID: The ID for the element.                                                 *;
%*            - type:  The recipient type.  Valid values: USER and GROUP.                          *;
%*            - principalId: The name of the recipient. The returned value is either a group       *;
%*              name or a user ID.                                                                 *;
%*            - description: The description of the recipient. The returned value is either a      *;
%*              group name or a user display name.                                                 *;
%*            - grpSrcCtxt: The context path in which the group recipient is defined, if the       *;
%*              recipient is a group.                                                              *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-12-13  original creation.                                                         *;
%* @history 2019-02-12  update to 2.1                                                              *;
%* @history 2019-03-15  rename from lsaf_getpfnotifrecipients to  lsaf_getpfsetupnotifrecips       *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                  *;
%*                                                                                                 *;

%macro lsaf_getpfsetupnotifrecips(
  lsaf_path=,
  lsaf_processflow=,
  sas_dsname=work.lsafgetpfsetupnotifrecips
  ) /des='Get a list of process flow notification setup element recipients for the specific process flow.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafgetpfsetupnotifrecips;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;
  %let notesOption=%sysfunc(getoption(notes));
  options notes;

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=processFlowPath processFlowName elementID
                         type principalId description grpSrcCtxt
          );
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300
           rowCount        length=8

           processFlowPath length=$2048  label="Context Path"
           processFlowName length=$256   label="Process Flow Name"
           elementID       length=$256   label="Element ID"
           type            length=$32    label="Principal Type"
           principalId     length=$256   label="Principal Name"
           description     length=$128   label="Principal Description"
           grpSrcCtxt      length=$2048  label="Group Context Source Path"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ProcessFlowSetupService");
    declare javaobj results("com/sas/lsaf/macro/results/SasProcessFlowElementsPrincipalsResult");

    %***********************************;
    %* Attempt the operation           *:
    %***********************************;
    srv.callStaticVoidMethod("getElementPrincipalsByType", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)",
         "%bquote(SYSTEM_NOTIFICATION)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process flow notification element recipients for &lsaf_processflow in &lsaf_path" ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Process flow notification element recipients were returned for &lsaf_processflow in &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getProcessFlowContext", processFlowPath);
        results.callStringMethod("getProcessFlowName", processFlowName);
        results.callStringMethod("getElementID", row, elementID);
        results.callStringMethod("getType", row, type);
        results.callStringMethod("getName", row, principalId);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getContextPath", row, grpSrcCtxt);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No recipients were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    options nonotes;
    proc sort data=&sas_dsname;
      by elementID type principalId;
    run;
  %end;
  options &notesOption;

%mend lsaf_getpfsetupnotifrecips;