%* lsaf_getquerytemplatedataset                                                                       *;
%*                                                                                                    *;
%* Gets the structure that is required as input to the macros that query the system. The metadata is  *;
%* stored in a SAS data set with zero observations.                                                   *;
%*                                                                                                    *;
%* Syntax:                                                                                            *;
%* %LSAF_GETQUERYTEMPLATEDATASET(<SAS_DSNAME=SAS-data-set>);                                          *;
%*                                                                                                    *;
%* @macvar _lsafRC_  The return code.                                                                 *;
%* @macvar _lsafMsg_ The return message.                                                              *;
%*                                                                                                    *;
%* @param sas_dsname - Optional - The name of the output SAS data set, specified as libref.dataset.   *;
%*            The default value is WORK.LSAFGETQUERYTEMPLATEDATASET. The data set contains no rows,   *;
%*            only the following columns that are required to submit a query:                         *;
%*            - recordType: The type of query element that the record represents.                     *;
%*            - columnClass: The class of the column to use with the query operation that is          *;
%*              associated with the record type.                                                      *;
%*            - columnName: The name of the column to use with the query operation that is            *;
%*              associated with the record type.                                                      *;
%*            - value: The value to use with the query operation that is associated with the record   *;
%*              type.                                                                                 *;
%*            - comparator: Indicates the operator that the query uses to handle the constraint.      *;
%*              This value is required when recordType is CONSTRAINT.                                 *;
%*            - isCaseSensitive: The Boolean value that indicates whether a column value is treated   *;
%*              as case-sensitive.                                                                    *;
%*                                                                                                    *;
%* @macrotype LSAF                                                                                    *;
%* @since 2.3                                                                                         *;
%* @exposure external                                                                                 *;
%*                                                                                                    *;
%* @history 2020-03-24  initial coding                                                                *;
%*                                                                                                    *;

%macro lsaf_getquerytemplatedataset(
  sas_dsname=work.lsafgetquerytemplatedataset
  ) /des='Get a query template data set for an LSAF query.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafgetquerytemplatedataset;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=recordType columnClass columnName value comparator isCaseSensitive 
    );

    attrib logMsgPrefix            length=$70;
    attrib logMsgPrimary           length=$70;
    attrib logMsg                  length=$300;

    attrib recordType              length=$32    label="Record Type";
    attrib columnClass             length=$32    label="Column Class";
    attrib columnName              length=$32    label="Column Name";
    attrib value                   length=$2048  label="Value Type";
    attrib comparator              length=$32    label="Comparator";
    attrib isCaseSensitive         length=8      label="Is Case Sensitive";
    
    recordType='';
    columnClass='';
    columnName='';
    value='';
    comparator='';
    isCaseSensitive=.;

    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    logMsgPrimary=logMsgPrefix;
    logMsg=" A query template data set was created.";

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafRC_= 0";
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    logMsg="Success";
    call symputx("_lsafRC_", 0);
    call symputx("_lsafMsg_", logMsg);
    call symputx("_lsafRowCount_", 0);

    if _n_ <1;

  run;

%mend lsaf_getquerytemplatedataset;
