%* lsaf_getrolemembers                                                                              *;
%*                                                                                                  *;
%* Gets the metadata for all of the members that are assigned to a role within the specified        *;
%* context. The metadata is stored in a SAS data set.                                               *;
%*                                                                                                  *;
%* Syntax:                                                                                          *;
%* %LSAF_GETROLEMEMBERS(LSAF_PATH=lsaf-path, LSAF_ROLE=lsaf-role-name,                              *;
%* LSAF_ROLE_CONTEXT=lsaf-role-context,  <, SAS_DSNAME=SAS-data-set>);                              *;
%*                                                                                                  *;
%* @macvar _lsafRC_  The return code.                                                               *;
%* @macvar _lsafMsg_ The return message.                                                            *;
%*                                                                                                  *;
%* @param lsaf_path - Required - The path of the context where the role is assigned.                *;
%* @param lsaf_role - Required - The name of the role.                                              *;
%* @param lsaf_role_context - Optional - The context in which the role is defined, specified as a   *;
%*            path (such as /SAS). If not provided, the members returned are from the role taht is  *;
%*            defined at the path.                                                                  *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata       *;
%*            for the members, specified as libref.dataset. The default value is                    *;
%*            WORK.LSAFGETROLEMEMBERS. The data set contains a row for each member in the role and  *;
%*            columns with the following names. The data set is sorted by member type and           *;
%*            principalId.                                                                          *;
%*            - contextPath: The path of the context.                                               *;
%*            - type: The member type. Valid values: user and group.                                *;
%*            - principalId: The name of the member. Valid values: group name and user ID.          *;
%*            - descriptor: The description of the member. Valid values: group name and user        *;
%*              display name.                                                                       *;
%*            - grpSrcCtxt: The context in which the group member is defined, if the member is      *;
%*              a group.                                                                            *;
%*                                                                                                  *;
%* @macrotype LSAF                                                                                  *;
%* @since 1.2                                                                                       *;
%* @exposure external                                                                               *;
%*                                                                                                  *;
%* @history 2013-08-30  updates for standardization and coding conventions                          *;
%* @history 2016-08-26  rebrand as LSAF                                                             *;
%* @history 2019-01-29  update to 2.1                                                               *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                   *;
%*                                                                                                  *;

%macro lsaf_getrolemembers(
  lsaf_path=,
  lsaf_role=,
  lsaf_role_context=,
  sas_dsname=work.lsafGetRoleMembers
  ) /des='Get a list of the members of a role in a specified context.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetRoleMembers;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=contextPath type principalId description grpSrcCtxt);
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib contextPath     length=$2048     label="Context Path" ;
    attrib type            length=$32       label="Principal Type";
    attrib principalId     length=$256      label="Principal Name";
    attrib description     length=$128      label="Principal Description";
    attrib grpSrcCtxt      length=$2048     label="Group Context Source Path";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/RoleService");
    declare javaobj results("com/sas/lsaf/macro/results/SasPrincipalsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getMembers", "%bquote(&lsaf_path)", "%bquote(&lsaf_role)", "%bquote(&lsaf_role_context)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve members for role &lsaf_role. defined for context path &lsaf_path.." ;
    end;
      else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Members were returned for role &lsaf_role. defined for context path &lsaf_path.." ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        contextPath="&lsaf_path";
        results.callStringMethod("getType", row, type);
        results.callStringMethod("getName", row, principalId);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getContextPath", row, grpSrcCtxt);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No members were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by principal id            *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by type principalId;
    run;
    options &notesOption;
  %end;

%mend lsaf_getrolemembers;
