%* lsaf_getroles                                                                                  *;
%*                                                                                                *;
%* Gets the metadata for all of the roles that are defined and inherited in a context. The        *;
%* metadata is stored in a SAS data set.                                                          *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETROLES(LSAF_PATH=lsaf-path <,SAS_DSNAME=SAS-data-set>);                                *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - Required - The path of the context in which the roles are located.          *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata     *;
%*            for the roles, specified as libref.dataset. The default value is                    *;
%*            WORK.LSAFGETROLES. The data set contains a row for each role in the context         *;
%*            and columns with the following names.  The data set is sorted by the role name.     *;
%*            - path: The path of the context.                                                    *;
%*            - name: The name of the role.                                                       *;
%*            - description: The role description.                                                *;
%*            - roleSrcCtxt: The context in which the of role is defined.                         *;
%*            - isInherited: Indicates whether the role is inherited. Valid values: 0 and 1.      *;
%* @macrotype LSAF                                                                                *;
%* @since 1.2                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-08-30  updates for standardization and coding conventions                        *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%* @history 2019-01-25  update to 2.1                     .                                       *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                 *;
%*                                                                                                *;

%macro lsaf_getroles(
  lsaf_path=,
  sas_dsname=work.lsafGetRoles
  ) /des='Get a list of roles in a SAS Life Science Analytics Framework context.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetRoles;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=contextPath name description roleSrcCtxt isInherited
  );
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib contextPath     length=$2048     label="Context Path" ;
    attrib name            length=$256      label="Role Name";
    attrib description     length=$128      label="Role Description";
    attrib roleSrcCtxt     length=$2048     label="Role Context Source Path";
    attrib isInherited     length=8         label="Is role inherited in current context: 0 or 1";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/RoleService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasRolesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getRoles", "%bquote(&lsaf_path)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve roles for &lsaf_path" ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Roles were returned for &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        contextPath="&lsaf_path";
        results.callStringMethod("getContextPath", row, roleSrcCtxt);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callIntMethod("isInherited", row, isInherited);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No roles were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by name                    *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by name;
    run;
    options &notesOption;
  %end;

%mend lsaf_getroles;
