%* lsaf_getschedulestatus                                                                         *;
%*                                                                                                *;
%* Gets the status of a schedule.                                                                 *;
%* SAS macro variable _lsafScheduleStatus_ is set to the status value. Values returned:           *;
%* - NORMAL: Active.                                                                              *;
%* - PAUSED: Paused.                                                                              *;
%* - ERROR: Error.                                                                                *;
%* - JOB_NOT_AVAILABLE: The job is not found at the specified path.                               *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETSCHEDULESTATUS(LSAF_SCHEDULEID=lsaf-scheduleid);                                      *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%* @macvar _lsafScheduleStatus_ The status of the schedule.                                       *;
%*                                                                                                *;
%* @param lsaf_scheduleid - Required - The case-sensitive identifier of the schedule. This can    *;
%*         be retrieved by calling either the lsaf_getallschedules or lsaf_getmyschedules macro.  *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 2.3                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2020-02-10  initial code                                                              *;
%*                                                                                                *;

%macro lsaf_getschedulestatus(
  lsaf_scheduleid=
  ) /des='Returns the status of a schedule.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_         
          _lsafRC_
          _lsafScheduleStatus_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib status          length=$25;

    status="";

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ScheduleService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("getScheduleStatus", "%bquote(&lsaf_scheduleid)", results, status);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine schedule status: &lsaf_scheduleid." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Returned schedule status: &lsaf_scheduleid." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafScheduleStatus_= " status;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafScheduleStatus_", strip(status) ) ;

  run;

%mend lsaf_getschedulestatus;
