%* lsaf_getscopedprivileges                                                                      *;
%*                                                                                               *;
%* Gets the metadata for all of the scoped privileges that are available. The metadata is        *;
%* stored in a SAS data set.                                                                     *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETSCOPEDPRIVILEGES(<SAS_DSNAME=SAS-data-set>);                                         *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the metadata    *;
%*            for the scoped privileges, specified as libref.dataset. The default value is       *;
%*            WORK.LSAFGETSCOPEDPRIVILEGES. The data set contains a row for each privilege       *;
%*            defined and columns with the following names. The data set is sorted by the        *;
%*            privilegeId.                                                                       *;
%*            - privilegeType: The privilege type: SCOPED                                        *;
%*            - privilegeId: The identifier of the privilege.                                    *;
%*            - privilegeName: The name of the privilege.                                        *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%* @history 2019-01-13  updates for LSAF 5.1 behavior                                            *;
%* @history 2019-01-13  renamed from lsaf_getcontextprivileges to lsaf_getscopedprivileges       *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                *;
%*                                                                                               *;

%macro lsaf_getscopedprivileges(
  sas_dsname=work.lsafGetScopedPrivileges
  )  /des='Get a list of the available scoped privileges.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetScopedPrivileges;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=privilegeType privilegeId privilegeName);
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib privilegeType   length=$20      label="Privilege Type" ;
    attrib privilegeId     length=$64      label="Privilege Identifier";
    attrib privilegeName   length=$128     label="Privilege Name";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ScopedPrivilegeService");
    declare javaobj results("com/sas/lsaf/macro/results/SasPrivilegesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getAvailablePrivileges", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve list of scoped privileges." ;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" List of scoped privileges was returned." ;
      results.callStringMethod("getType", privilegeType);
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getId", row, privilegeId);
        results.callStringMethod("getName", row, privilegeName);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by privilege id            *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by privilegeId;
    run;
    options &notesOption;
  %end;

%mend lsaf_getscopedprivileges;
