%* lsaf_getsystemconstants                                                                         *;
%*                                                                                                 *;
%* Gets a list of the SAS Life Science Analytics Framework constants that are used as a parameter, *;
%* data set, or return value. The metadata is stored in a SAS data set.                            *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETSYSTEMCONSTANTS(SAS_DSNAME=SAS-data-set);                                              *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the data. The     *;
%*            default value is WORK.LSAFGETSYSTEMCONSTANTS. The data set contains a row for each   *;
%*            constant and columns with the following names:                                       *;
%*            - MacroApiVersion: The version of the SAS Macro API that the list pertains to.       *;
%*            - Service: The macro service that uses or returns the constant.                      *;
%*            - Type: The type of the constant value.                                              *;
%*            - Name: The name of the constant.                                                    *;
%*            - Value: The value of the constant, if different than the name.                      *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 2.1                                                                                      *;
%* @exposure internal                                                                              *;
%*                                                                                                 *;
%* @history 2019-01-24  initial code                                                               *;
%* @history 2019-10-02  added macro api version release to data set                                *;
%* @history 2019-11-01  update to handle a blank SAS output file.                                  *;
%*                                                                                                 *;

%macro lsaf_getsystemconstants(
  sas_dsname=work.lsafGetSystemConstants
  ) /des='Get a list of the system constants';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetSystemConstants;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= macroApiVersion service type name value);

    attrib returnCode              length=8;
    attrib message                 length=$200;
    attrib logMsgPrefix            length=$70;
    attrib logMsgPrimary           length=$70;
    attrib logMsg                  length=$300;
    attrib rowCount                length=8;

	attrib macroApiVersion         length=$30    label="Macro API version";
    attrib service                 length=$64    label="Service";
    attrib type                    length=$64    label="Type";
    attrib name                    length=$64    label="Name";
    attrib value                   length=$100   label="Value";

    rowCount=0;                                                                                                                                       

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/SystemService");
    declare javaobj results("com/sas/lsaf/macro/results/SasSystemConstantsResult");

    %***********************************;
    %* Attempt the operation           *:
    %***********************************;
    srv.callStaticVoidMethod("getLSAFConstants", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve list of constants.";
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" List of constants was returned.";
	  results.callStringMethod("getMacroRelease", macroApiVersion);
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getService", row, service);
        results.callStringMethod("getType", row, type);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getValue", row, value);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0)
      then put logMsgPrefix " No constants were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;
  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    proc sort data=&sas_dsname;
      by service type value;
    run;
  %end;

  options &notesOption;

%mend lsaf_getsystemconstants;
