%* lsaf_getuserproperties                                                                           *;
%*                                                                                                  *;
%* Gets all of the properties for the specified user. The properties are stored in a SAS data set.  *;
%*                                                                                                  *;
%* Syntax:                                                                                          *;
%* %LSAF_GETUSERPROPERTIES(LSAF_USERID=lsaf-userid <,SAS_DSNAME=SAS-data-set>);                     *;
%*                                                                                                  *;
%* @macvar _lsafRC_  The return code.                                                               *;
%* @macvar _lsafMsg_ The return message.                                                            *;
%*                                                                                                  *;
%* @param lsaf_userid - Required - The identifier of the user.                                      *;
%* @param sas_dsname - Optional - The name of the output SAS data set to contain the properties     *;
%*            for the user, specified as libref.dataset.  The default value is                      *;
%*            WORK.LSAFGETUSERPROPERTIES.  The data set is sorted by name and contains the          *;
%*            following columns:                                                                    *;
%*            - userId: The unique login identifier for the user.                                   *;
%*            - name: The name of the property.                                                     *;
%*            - value: The value of the property.                                                   *;
%*            - displayName: The name of the property as displayed as a label within SAS Life       *;
%*              Science Analytics Framework.                                                        *;
%*            - type: The type of the property. Valid values: STRING, DATE, and BOOLEAN.            *;
%*            - isReadonly: Indicates whether the property cannot be modified. Valid values:        *;
%*              - 0: No.                                                                            *;
%*              - 1: Yes.                                                                           *;
%*            - isExtended: Indicates whether the property is an extended attribute. Valid values:  *;
%*              - 0: No.                                                                            *;
%*              - 1: Yes.                                                                           *;
%*            - isRequired: Indicates whether the property is required. Valid values:               *;
%*              - 0: No.                                                                            *;
%*              - 1: Yes.                                                                           *;
%*                                                                                                  *;
%* @macrotype LSAF                                                                                  *;
%* @since 2.3                                                                                       *;
%* @exposure external                                                                               *;
%*                                                                                                  *;
%* @history 2019-12-10  initial coding                                                              *;
%*                                                                                                  *;

%macro lsaf_getuserproperties(
  lsaf_userid=,
  sas_dsname=work.lsafGetUserProperties
  ) /des='Get a list of user properties.';
  
  %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetUserProperties;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= userId name displayName type value isReadOnly isRequired isExtended
  );

    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib userId          length=$64    label="User ID";
    attrib name            length=$256   label="Property Name";
    attrib displayName     length=$256   label="Property Display Name";
    attrib value           length=$2048  label="Property Value";
    attrib type            length=$32    label="Property Type";
    attrib isReadonly      length=8      label="Property value is not modifiable.";
    attrib isExtended      length=8      label="Property is an Extended Attribute";
    attrib isRequired      length=8      label="Property is Required";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/UserService") ;
    declare javaobj results("com/sas/lsaf/macro/results/SasObjectPropertiesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getProperties", "%bquote(&lsaf_userid)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve list of user properties.";
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" List of user properties was returned.";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        userid="&lsaf_userid";
        results.callStringMethod("getPropertyName", row, name);
        results.callStringMethod("getPropertyDisplayName", row, displayName);
        results.callStringMethod("getPropertyType", row, type);
        results.callStringMethod("getPropertyValue", row,value);
        results.callBooleanMethod("getPropertyIsReadonly", row, isReadOnly);
        results.callBooleanMethod("getPropertyIsRequired", row, isRequired);
        results.callBooleanMethod("getPropertyIsExtended", row, isExtended);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    if (rowCount=0) 
      then put logMsgPrefix " No user properties were found.";
      else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by name;
    run;
    options &notesOption;
  %end;

%mend lsaf_getuserproperties;