%* lsaf_isjobscheduled                                                                      *;
%*                                                                                          *;
%* Indicates whether a job is scheduled.                                                    *;
%*                                                                                          *;
%* The SAS macro variable _lsafJobIsScheduled_ is set to a value of 1 when the job is       *;
%* scheduled, and 0 when it is not. In the event of an incorrect call to the macro          *;
%* that results in an error, the return value of _lsafJobIsScheduled_ is set to -1.         *;
%*                                                                                          *;
%* This macro returns False, if you do not have access to the schedule. Access to the       *;
%* schedules is controlled by the global privileges PRIVILEGE_MANAGE_SCHEDULES and          *;
%* PRIVILEGE_MANAGE_ALL_SCHEDULES.                                                          *;
%*                                                                                          *;
%* Syntax:                                                                                  *;
%* %LSAF_ISJOBSCHEDULED(LSAF_LOCATION=lsaf-location, LSAF_PATH=lsaf-path);                  *;
%*                                                                                          *;
%* @macvar _lsafRC_  The return code.                                                       *;
%* @macvar _lsafMsg_ The return message.                                                    *;
%* @macvar _lsafJobIsScheduled_ Indicates whether the job is scheduled.                     *;
%*                                                                                          *;
%* @param  lsaf_location - Required - The case-insensitive location of the job.             *;
%*     Valid values: WORKSPACE and REPOSITORY.                                              *;
%* @param lsaf_path - Required - The path of the job.                                       *;
%*                                                                                          *;
%* @macrotype LSAF                                                                          *;
%* @since 2.3                                                                               *;
%* @exposure external                                                                       *;
%*                                                                                          *;
%* @history 2019-11-20  initial code                                                        *;
%*                                                                                          *;

%macro lsaf_isjobscheduled(
  lsaf_location=,
  lsaf_path=
  ) /des='Is the job scheduled?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getproductname();

  %global _lsafMsg_         
          _lsafRC_
          _lsafJobIsScheduled_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib scheduled          length=3;
    attrib logMsgPrimary   length=$70;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ScheduleService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isJobScheduled", "%bquote(&lsaf_location)", "%bquote(&lsaf_path)", results, scheduled);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
	%let lsaf_location = %UPCASE(&lsaf_location.);
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the &lsaf_location job is scheduled: &lsaf_path." ;
      scheduled=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (scheduled) then
      do;
        logMsg=" The &lsaf_location job is scheduled: &lsaf_path." ;
      end;
      else
      do;
        logMsg=" The &lsaf_location job is not scheduled: &lsaf_path." ;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafJobIsScheduled_= " scheduled;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafJobIsScheduled_", scheduled) ;

  run;

%mend lsaf_isjobscheduled ;
