%* lsaf_isversioned                                                                                      *;
%*                                                                                                       *;
%* Indicates whether a repository file is versioned.                                                     *;
%*                                                                                                       *;
%* The SAS macro variable _lsafIsVersioned_ is set to a value of 1, if the file is versioned,            *;
%* 0 if it is not. In the event of an incorrect call to the macro that results in an error, the          *;
%* the return value of _lsafIsVersioned_ will be set to -1.                                              *;
%*                                                                                                       *;
%* Syntax:                                                                                               *;
%* %LSAF_ISVERSIONED(LSAF_PATH=lsaf-path);                                                               *;
%*                                                                                                       *;
%* @macvar _lsafRC_  The return code.                                                                    *;
%* @macvar _lsafMsg_ The return message.                                                                 *;
%* @macvar _lsafIsVersioned_ Indicates whether the file is versioned.                                    *;
%*                                                                                                       *;
%* @param lsaf_path - Required - The path to the file.                                                   *;
%*                                                                                                       *;
%* @macrotype LSAF                                                                                       *;
%* @since 1.1                                                                                            *;
%* @exposure external                                                                                    *;
%*                                                                                                       *;
%* @history 2013-08-30  updates for standardization and coding conventions                               *;
%* @history 2016-08-26  rebrand as LSAF                                                                  *;
%* @history 2018-01-09  updates to documentation for LSAF 5.1                                            *;
%*                                                                                                       *;

%macro lsaf_isversioned(
  lsaf_path=
  ) /des='Does the file in the repository have versioning enabled?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafIsVersioned_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;
    attrib isVersioned     length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isVersioned", "%bquote(&lsaf_path)", results, isVersioned);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the file is versioned: &lsaf_path." ;
      isVersioned=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (isVersioned) then
      do;
        logMsg=" The file is versioned: &lsaf_path.";
      end;
      else
      do;
        logMsg=" The file is not versioned: &lsaf_path.";
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafIsVersioned_= " isVersioned;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafIsVersioned_", isVersioned) ;

  run;

%mend lsaf_isversioned ;
