%* lsaf_move                                                                                     *;
%*                                                                                               *;
%* Moves a file or folder to the specified target location. Contexts cannot be moved.            *;
%*                                                                                               *;
%* The item to move must not exist in the target location. The children of a moved folder are    *;
%* also moved to the target location, keeping their directory structure and version history      *;
%* (if any) intact.                        *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_MOVE(LSAF_SOURCE=lsaf-source-path, LSAF_TARGET=lsaf-target-path);                       *;
%*                                                                                               *;
%* Example Code:                                                                                 *;                                                                                                                                                                                                                   
%*                                                                                               *;                                                                                                                                                                                                                   
%*    %lsaf_move(lsaf_source=%str(/contextName/srcFolder/scrFile.txt),                           *;                                                                                                                                                                                                                   
%*               lsaf_target=%str(/contextName/destFolder/scrFile.txt));                         *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_source - Required - The path of the file or folder.                               *;
%* @param lsaf_target - Required - The full path to which to move the file or folder.            *;
%*                      The target location must exist. The path cannot end with a slash (/).    *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.4                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2014-03-03  initial coding                                                           *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%* @history 2018-12-10  documentation update for LSAF 5.x behavior                               *;
%*                                                                                               *;

%macro lsaf_move(
  lsaf_source=,
  lsaf_target=
  ) /des='Move a file or folder.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/RepositoryService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("move", "%bquote(&lsaf_source)", "%bquote(&lsaf_target)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The move operation was successful.";
    end;
    else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot successfully move the item.  Target was not created." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " lsaf_source = &lsaf_source" ;
    put logMsgPrefix " lsaf_target = &lsaf_target" ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_move ;
