%* lsaf_process_jobdatasets                                                                          *;
%*                                                                                                   *;
%* Validates the data sets that are specified by the user to create or update jobs. The data sets    *;
%* are combined into one central data set for processing.                                          *;
%*                                                                                                   *;
%* Syntax:                                                                                           *;
%* %LSAF_PROCESS_JOBDATASETS(CALLINGMACRO=calling-macro <, sas_dsname_info,                          *;
%*     sas_dsname_programs, sas_dsname_parameters, sas_dsname_inputs, sas_dsname_outputs> );         *;
%*                                                                                                   *;
%* @macvar _lsafRC_  the return code.                                                                *;
%* @macvar _lsafMsg_ the return message.                                                             *;
%*                                                                                                   *;
%* @macrotype LSAF                                                                                   *;
%* @since 1.5                                                                                        *;
%* @exposure internal                                                                                *;
%*                                                                                                   *;
%* @history 2016-09-26  initial coding                                                               *;
%* @history 2016-11-07  modifications for update jobs                                                *;
%* @history 2019-02-27  modifications for 2.1                                                        *;
%*                                                                                                   *;

%macro lsaf_process_jobdatasets(
   callingMacro=,
   sas_dsname_info=,
   sas_dsname_programs=,
   sas_dsname_parameters=,
   sas_dsname_inputs=,
   sas_dsname_outputs=
   ) / des='Validates SAS data sets for use in creating and updating jobs.';

   %let sas_dsname_info=%CMPRES(&sas_dsname_info);
   %let sas_dsname_programs=%CMPRES(&sas_dsname_programs);
   %let sas_dsname_parameters=%CMPRES(&sas_dsname_parameters);
   %let sas_dsname_inputs=%CMPRES(&sas_dsname_inputs);
   %let sas_dsname_outputs=%CMPRES(&sas_dsname_outputs);

   %let notprovidedstring=%str(_&callingMacro._ds_notprovided_);
   %if ( %index(%upcase(&callingMacro), UPDATE) NE 0) %then
      %let updating=1;
   %else
      %let updating=0;

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %let _lsafRC_=%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %global _lsafMASTERJobDATA_tempDS_
           source_info
           source_programs
           source_parms
           source_inputs
           source_outputs
   ;

   %let _lsafMASTERJobDATA_tempDS_=%str(_null_);
   %let _lsaf_tempDS_=%str(_null_);
   %let _tempINPUTS_tempDS_=%str(_null_);
   %let _tempOUTPUTS_tempDS_=%str(_null_);
   %let _tempPARAMETERS_tempDS_=%str(_null_);

   %if (&sas_dsname_info=&notprovidedstring AND &sas_dsname_programs=&notprovidedstring AND
        &sas_dsname_parameters=&notprovidedstring AND &sas_dsname_inputs=&notprovidedstring AND
        &sas_dsname_outputs=&notprovidedstring) %then
      %do;
         %if (&updating) %then
            %let _lsafMsg_= %str(No data has been provided for the update.);
         %else
            %let _lsafRC_=0;
         %RETURN;
      %end;

   %let source_info=INFO;
   %let source_programs=PROGRAMS;
   %let source_parms=PARMS;
   %let source_inputs=INPUTS;
   %let source_outputs=OUTPUTS;


   %let _lsafMASTERJobDATA_tempDS_=%str(__LSAF_SASMacro_createjobtemp__);
   %let _lsaf_tempDS_=%str(__LSAF_temp__);

   %*************************************************************;
   %* Turn off the notes generated for all the pre-processing.  *;
   %* Save original option setting so it can be reset properly. *;
   %*************************************************************;

   options nonotes;

   %put NOTE: &macroName: Processing job data sets.;

   %***************************************************************************;
   %* Cleanup from any previous runs that may have left temp data set behind. *;
   %***************************************************************************;
   %let _lsafMASTERJobDATA_tempDS_=%str(__LSAF_SASMacro_processjobtemp__);
   %let _lsaf_tempDS_=%str(__LSAF_temp__);

   %if (%sysfunc(exist(&_lsafMASTERJobDATA_tempDS_)) > 0) %then
   %do;
      %put NOTE: &macroName: Deleting temporary data set &_lsafMASTERJobDATA_tempDS_ before processing.;
      proc datasets library=work nolist;
         delete &_lsafMASTERJobDATA_tempDS_;
      quit;
      run;
   %end;

   %if (%sysfunc(exist(&_lsaf_tempDS_)) > 0) %then
   %do;
      %put NOTE: &macroName: Deleting temporary data set &_lsaf_tempDS_ before processing.;
      proc datasets library=work nolist;
         delete &_lsaf_tempDS_;
      quit;
      run;
   %end;

   %local _lsaf_tempDS_
          datasetstructure
          datasetvars
          notprovidedstring
          updating
   ;

   %let datasetvars = %str(source name value version pLabel type includeSubFolders);
   %let datasetstructure =
      attrib source              length=$20
             name                length=$256
             value               length=$2048
             version             length=$32
             pLabel              length=$256
             type                length=$32
             includeSubFolders   length=8
      ;
   data &_lsafMASTERJobDATA_tempDS_;
      &datasetstructure;
      STOP;
   run;

   %*********************************************************************************;
   %* JOB INFO data set - if supplied, validate it and add it to master data set.   *;
   %*********************************************************************************;
   %if ( &sas_dsname_info NE &notprovidedstring ) %then
   %do;
      options notes;
      %put NOTE: &callingMacro.: Validating JOB INFORMATION data set: &sas_dsname_info;
      %lsaf_validate_sasdataset(callingMacro=&macroName, sas_dsn=&sas_dsname_info, required_variables=%str(name value),
                                allow_empty_datasets=&updating);
      %if (&_lsafRC_ ne 0) %then
      %do;
         %let _lsafMsg_= sas_dsname_info: &_lsafMsg_;
         %RETURN;
      %end;

      %******************************************;
      %* JOB INFO - add to master data set.     *;
      %******************************************;
      options nonotes;
      data &_lsaf_tempDS_ (keep=&datasetvars);
         &datasetstructure;
         set &sas_dsname_info;
         source="&source_info";
      run;
      proc append base=&_lsafMASTERJobDATA_tempDS_ data=&_lsaf_tempDS_;
      run;
   %end;

   %*********************************************************************************;
   %* PARAMETERS data set - if supplied, validate it and add it to master data set. *;
   %*********************************************************************************;
   %if ( &sas_dsname_parameters NE &notprovidedstring ) %then
   %do;
      options notes;
      %put NOTE: &callingMacro.: Validating JOB PARAMETERS data set: &sas_dsname_parameters;
      %lsaf_validate_sasdataset(callingMacro=&macroName, sas_dsn=&sas_dsname_parameters,
                                required_variables=%str(name label type defaultValue fileVersion includeSubFolders),
                                required_numeric_variables=%str(includeSubFolders),
                                allow_empty_datasets=&updating);
      %if (&_lsafRC_ ne 0) %then
      %do;
         %let _lsafMsg_= sas_dsname_parameters: &_lsafMsg_;
         %RETURN;
      %end;

      %*****************************************************************;
      %* PARAMETERS: Validate that no duplicate values are present.    *;
      %* Order of original data set needs to be preserved.             *;
      %*****************************************************************;
      %put NOTE: &callingMacro.: Testing for duplicate parameters;
      %let _tempPARAMETERS_tempDS_=%str(_tempPARAMETERS_tempDS_);
      %if (%sysfunc(exist(&_tempPARAMETERS_tempDS_)) > 0) %then
      %do;
         %put NOTE: %sysfunc(lowcase(&SYSMACRONAME)): Deleting temporary data set &_tempPARAMETERS_tempDS_ before processing.;
         proc datasets library=work nolist;
            delete &_tempPARAMETERS_tempDS_;
         quit;
         run;
      %end;
      proc sort data=&sas_dsname_parameters out=&_tempPARAMETERS_tempDS_;
         by name;
      run;
      data _null_;
         set &_tempPARAMETERS_tempDS_;
         by name;
         if (first.name AND NOT(last.name)) then
         do;
            call symput("_lsafRC_", "-999");
            call symput("_lsafMsg_", "sas_dsname_parameters: Data set has duplicate values for parameter name.");
            STOP;
         end;
      run;
      %if (&_lsafRC_ ne 0) %then
      %do;
         %RETURN;
      %end;

      %******************************************;
      %* PARAMETERS - add to master data set.   *;
      %******************************************;
      options nonotes;
      data &_lsaf_tempDS_ (keep=&datasetvars);
         &datasetstructure;
         set &sas_dsname_parameters;
         source="&source_parms";
         pLabel=label;
         value=defaultValue;
         version=fileVersion;
      run;

      proc append base=&_lsafMASTERJobDATA_tempDS_ data=&_lsaf_tempDS_;
      run;

   %end;

   %********************************************************************************;
   %* PROGRAMS data set - if supplied, validate it and add it to master data set.  *;
   %********************************************************************************;
   %if ( &sas_dsname_programs NE &notprovidedstring ) %then
   %do;
      options notes;
      %put NOTE: &callingMacro.: Validating JOB PROGRAMS data set: &sas_dsname_programs;
      %lsaf_validate_sasdataset(callingMacro=&macroName, sas_dsn=&sas_dsname_programs, required_variables=%str(programPath programVersion),
                                allow_empty_datasets=&updating);
      %if (&_lsafRC_ ne 0) %then
      %do;
         %let _lsafMsg_= sas_dsname_programs: &_lsafMsg_;
         %RETURN;
      %end;

      %**************************************;
      %* PROGRAMS - add to master data set. *;
      %**************************************;
      options nonotes;
      data &_lsaf_tempDS_ (keep=&datasetvars);
         &datasetstructure;
         set &sas_dsname_programs (rename=(programPath=value programVersion=version));
         source="&source_programs";
      run;
      proc append base=&_lsafMASTERJobDATA_tempDS_ data=&_lsaf_tempDS_;
      run;
   %end;

   %*********************************************************************************;
   %* INPUTS data set - if supplied, validate it and add it to master data set.     *;
   %*********************************************************************************;
   %if ( &sas_dsname_inputs NE &notprovidedstring ) %then
   %do;
      options notes;
      %put NOTE: &callingMacro.: Validating JOB INPUTS data set: &sas_dsname_inputs;
      %lsaf_validate_sasdataset(callingMacro=&macroName, sas_dsn=&sas_dsname_inputs, required_variables=%str(inputPath inputType inputVersion inputIncludeSubFolders),
                                required_numeric_variables=%str(inputIncludeSubFolders),
                                allow_empty_datasets=&updating);
      %if (&_lsafRC_ ne 0) %then
      %do;
         %let _lsafMsg_= sas_dsname_inputs: &_lsafMsg_;
         %RETURN;
      %end;

      %*****************************************************************;
      %* INPUTS: Validate that no duplicate values are present.        *;
      %* Order of original data set needs to be preserved.             *;
      %*****************************************************************;
      %put NOTE: &callingMacro.: Testing for duplicate input paths;
      %let _tempINPUTS_tempDS_=%str(_tempINPUTS_tempDS_);
      %if (%sysfunc(exist(&_tempINPUTS_tempDS_)) > 0) %then
      %do;
         %put NOTE: %sysfunc(lowcase(&SYSMACRONAME)): Deleting temporary data set &_tempINPUTS_tempDS_ before processing.;
         proc datasets library=work nolist;
            delete &_tempINPUTS_tempDS_;
         quit;
         run;
      %end;
      proc sort data=&sas_dsname_inputs out=&_tempINPUTS_tempDS_;
         by inputPath;
      run;
      data _null_;
         set &_tempINPUTS_tempDS_;
         by inputPath;
         if (first.inputPath AND NOT(last.inputPath)) then
         do;
            call symput("_lsafRC_", "-999");
            call symput("_lsafMsg_", "sas_dsname_inputs: Data set has duplicate values for inputPath.");
            STOP;
         end;
      run;
      %if (&_lsafRC_ ne 0) %then
      %do;
         %RETURN;
      %end;

      %******************************************;
      %* INPUTS - add to master data set.       *;
      %******************************************;
      options nonotes;
      data &_lsaf_tempDS_ (keep=&datasetvars);
         &datasetstructure;
         set &sas_dsname_inputs (rename=(inputPath=value inputVersion=version inputType=type inputIncludeSubFolders=includeSubFolders));
         source="&source_inputs";
      run;
      proc append base=&_lsafMASTERJobDATA_tempDS_ data=&_lsaf_tempDS_;
      run;
   %end;

   %*********************************************************************************;
   %* OUTPUTS data set - if supplied, validate it and add it to master data set.     *;
   %*********************************************************************************;
   %if ( &sas_dsname_outputs NE &notprovidedstring ) %then
   %do;
      options notes;
      %put NOTE: &callingMacro.: Validating JOB OUTPUTS data set: &sas_dsname_outputs;
      %lsaf_validate_sasdataset(callingMacro=&macroName, sas_dsn=&sas_dsname_outputs, required_variables=%str(outputPath outputIncludeSubFolders),
                                required_numeric_variables=%str(outputIncludeSubFolders),
                                allow_empty_datasets=&updating);
      %if (&_lsafRC_ ne 0) %then
      %do;
         %let _lsafMsg_= sas_dsname_outputs: &_lsafMsg_;
         %RETURN;
      %end;

      %*****************************************************************;
      %* OUTPUTS: Validate that no duplicate values are present.       *;
      %* Order of original data set needs to be preserved.             *;
      %*****************************************************************;
      %put NOTE: &callingMacro.: Testing for duplicate output paths;
      %let _tempOUTPUTS_tempDS_=%str(_tempOUTPUTS_tempDS_);
      %if (%sysfunc(exist(&_tempOUTPUTS_tempDS_)) > 0) %then
      %do;
         %put NOTE: %sysfunc(lowcase(&SYSMACRONAME)): Deleting temporary data set &_tempOUTPUTS_tempDS_ before processing.;
         proc datasets library=work nolist;
            delete &_tempOUTPUTS_tempDS_;
         quit;
         run;
      %end;
      proc sort data=&sas_dsname_outputs out=&_tempOUTPUTS_tempDS_;
         by outputPath;
      run;
      data _null_;
         set &_tempOUTPUTS_tempDS_;
         by outputPath;
         if (first.outputPath AND NOT(last.outputPath)) then
         do;
            call symput("_lsafRC_", "-999");
            call symput("_lsafMsg_", "sas_dsname_outputs: Data set has duplicate values for outputPath.");
            STOP;
         end;
      run;
      %if (&_lsafRC_ ne 0) %then
      %do;
         %RETURN;
      %end;

      %******************************************;
      %* OUTPUTS - add to master data set.       *;
      %******************************************;
      options nonotes;
      data &_lsaf_tempDS_ (keep=&datasetvars);
         &datasetstructure;
         set &sas_dsname_outputs (rename=(outputPath=value outputIncludeSubFolders=includeSubFolders));
         source="&source_outputs";
      run;
      proc append base=&_lsafMASTERJobDATA_tempDS_ data=&_lsaf_tempDS_;
      run;
   %end;

   %let _lsafRC_=0;
   
   %***************************************************************************;
   %* Cleanup temp data sets.                                                 *;
   %***************************************************************************;
   proc datasets library=work nolist;
      delete __LSAF_SASMACRO_PROCESSJOBTEMP_;
   quit;
   run;

   %if (%sysfunc(exist(&_lsaf_tempDS_)) > 0) %then
   %do;
      proc datasets library=work nolist;
         delete &_lsaf_tempDS_;
      quit;
      run;
   %end;
   
   %if (%sysfunc(exist(&_tempINPUTS_tempDS_)) > 0) %then
   %do;
      proc datasets library=work nolist;
         delete &_tempINPUTS_tempDS_;
      quit;
      run;
   %end;
   
   %if (%sysfunc(exist(&_tempOUTPUTS_tempDS_)) > 0) %then
   %do;
      proc datasets library=work nolist;
         delete &_tempOUTPUTS_tempDS_;
      quit;
      run;
   %end;
   
   %if (%sysfunc(exist(&_tempPARAMETERS_tempDS_)) > 0) %then
   %do;
      proc datasets library=work nolist;
         delete &_tempPARAMETERS_tempDS_;
      quit;
      run;
   %end;

   options &notesOption;

%mend lsaf_process_jobdatasets;
