%* lsaf_processdefdeployedatcontext                                                                      *;
%*                                                                                                       *;
%* Indicates whether a process flow definition is deployed at the specified context type.                *;
%*                                                                                                       *;
%* The macro variable _lsafPFDefDeployedAtContext_ is set to a value of 1 when the process flow is       *;
%* deployed for the context type, 0 if it is not. In the event of an incorrect call to the macro that    *;
%* results in an error, the return value of _lsafPFDefDeployedAtContext_ is set to -1.                   *;
%*                                                                                                       *;
%* Syntax:                                                                                               *;
%* %LSAF_PROCESSDEFDEPLOYEDATCONTEXT(LSAF_processflowdef=process-flow-definition-name,                   *;
%*                                   LSAF_TYPEID=lsaf-typeid);                                           *;
%*                                                                                                       *;
%* @macvar _lsafRC_  The return code.                                                                    *;
%* @macvar _lsafMsg_ The return message.                                                                 *;
%* @macvar _lsafPFDefDeployedAtContext_ Indicates whether the process flow definition is deployed at the *;
%*          specified context.                                                                           *;
%*                                                                                                       *;
%* @param lsaf_processflowdef - Required - The ID of the process flow definition.                        *;
%* @param lsaf_typeid - Required - The context type. The macro lsaf_gettype can be used to determine     *;
%*            the type ID for the context.                                                               *;
%*                                                                                                       *;
%* @macrotype LSAF                                                                                       *;
%* @since 2.1                                                                                            *;
%* @exposure external                                                                                    *;
%*                                                                                                       *;
%* @history 2019-03-12  Initial coding                                                                   *;
%*                                                                                                       *;

%macro lsaf_processdefdeployedatcontext(
  lsaf_processflowdef=,
  lsaf_typeid=
  ) /des='Has the process flow definition been deployed for the specified context type?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafPFDefDeployedAtContext_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsgPrimary   length=$70
           logMsg          length=$300
           isDeployed      length=3
           ;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ProcessDefinitionService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isProcessDefinitionDeployedForContextType", "%bquote(&lsaf_processflowdef)", "%bquote(&lsaf_typeid)", results, isDeployed);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the process flow definition: &lsaf_processflowdef is deployed for the specified context: &lsaf_typeid.." ;
      isDeployed=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (isDeployed) 
        then logMsg=" The process flow definition: &lsaf_processflowdef is deployed at context: &lsaf_typeid..";
        else logMsg=" The process flow definition: &lsaf_processflowdef is not deployed at context: &lsaf_typeid..";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafPFDefDeployedAtContext_= " isDeployed;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafPFDefDeployedAtContext_", isDeployed) ;

  run;

%mend lsaf_processdefdeployedatcontext;
