%* lsaf_queryauditentries                                                                                   *;
%*                                                                                                          *;
%* Extracts audit entries from the audit history records. The entries to extract are specified in a query   *;
%* that is contained in a data set. The extracted data is stored in a comma-separated values file that      *;
%* is located in your workspace or in the repository.                                                       *;  
%*                                                                                                          *;
%* Information that concerns the necessary columnClass and columnName data, as well as their data types,    *;
%* can be retrieved by calling the macro lsaf_getquerycolumns with the type AuditEntryQuery.                *;
%*                                                                                                          *;
%* You must have the View Audit History privilege to execute the query.                                     *;
%*                                                                                                          *;
%* See the SAS Life Science Analytics Framework Macros API Users Guide for examples and the Module          *;
%* QueryService header text above for a detailed explanation of the macro parameters.                       *;
%*                                                                                                          *;
%* Syntax:                                                                                                  *;
%* %LSAF_QUERYAUDITENTRIES(LSAF_QUERYDATASET=lsaf-querydataset,                                             *; 
%*     LSAF_VALIDATEDDATASET=lsaf-validateddataset,                                                         *;
%*     LSAF_EXPORTLOCATION=lsaf-exportlocation,                                                             *;
%*     LSAF_EXPORTPATH=lsaf-exportpath,                                                                     *;
%*     LSAF_OVERWRITE=lsaf-overwrite,                                                                       *; 
%*     LSAF_ENABLEVERSIONING=lsaf-enableversioning,                                                         *; 
%*     LSAF_VERSIONTYPE=lsaf-versiontype,                                                                   *; 
%*     LSAF_CUSTOMVERSION=lsaf-customversion,                                                               *; 
%*     LSAF_COMMENT=lsaf-comment);                                                                          *;
%*                                                                                                          *;
%* @macvar _lsafRC_  The return code.                                                                       *;
%* @macvar _lsafMsg_ The return message.                                                                    *;
%* @macvar _lsafAuditDetailsExportPath_ The full path to the query results .csv file, which includes        *;
%*            the file extension. If the query is not executed or if the macro processing fails,            *;
%*            _lsafAuditDetailsExportPath_ is blank.                                                        *;
%*                                                                                                          *;
%* @param lsaf_querydataset - Required - The name of the input SAS data set that contains the query         *;
%*            metadata, specified as libref.dataset. When querying against the columnName MODE with the     *:
%*            columnClass of AuditEntry, the valid values are USER, SYSTEM, and ADMIN.                      *;
%* @param lsaf_validateddataset - Required - The name of the output SAS data set to contain the             *;
%*            results of the validation of the records from the input data set lsaf_querydataset,           *;
%*            specified as libref.dataset. The default data set name is WORK.LSAFQUERYAUDITENTRIES.         *;
%* @param lsaf_exportlocation - Required - The case-insensitive output location for the exported file.      *;
%*            Valid values: REPOSITORY and WORKSPACE.                                                       *;
%* @param lsaf_exportpath - Required - The case-sensitive output path for the exported CSV file.            *;
%* @param lsaf_overwrite - Optional - Indicates whether an existing nonversioned file is overwritten.       *;
%*            Valid values: 0 (Default, No) and 1 (Yes).                                                    *;
%* @param lsaf_enableversioning - Optional - Indicates whether versioning is enabled for a new file         *;
%*            in the repository. When lsaf_exportlocation=WORKSPACE, this value is ignored.                 *;
%*            Valid values: 0 (Default, No) and 1 (Yes).                                                    *; 
%* @param lsaf_versiontype - Optional - The version type to use to create a file in the repository. When    *;
%*            lsaf_exportlocation is WORKSPACE or lsaf_enableversioning is FALSE, this value is ignored.    *;
%*            Valid values: MAJOR (Default), MINOR, and CUSTOM.                                             *;
%* @param lsaf_customversion - Conditional - The version number to use to create a customized versioned     *;
%*            file in the repository. When lsaf_exportlocation is WORKSPACE, lsaf_enableversioning is       *;
%*            FALSE, or lsaf_versiontype is not CUSTOM, this value is ignored.                              *;
%* @param lsaf_comment - Optional - The check-in comment to associate with the action of adding a file      *;
%*            to the repository. When lsaf_exportlocation=WORKSPACE, this value is ignored.                 *;
%*                                                                                                          *;
%* @macrotype LSAF                                                                                          *;
%* @since 2.3                                                                                               *;
%* @exposure external                                                                                       *;
%*                                                                                                          *;
%* @history 2020-03-03  initial coding                                                                      *;
%*                                                                                                          *;

%macro lsaf_queryauditentries(
  lsaf_querydataset=,
  lsaf_validateddataset=work.lsafqueryauditentries,
  lsaf_exportlocation=,
  lsaf_exportpath=,
  lsaf_overwrite=0,
  lsaf_enableversioning=0,
  lsaf_versiontype=MAJOR,
  lsaf_customversion=,
  lsaf_comment=
  ) /des='Performs a query and returns an output file to a specified location in the SAS Life Science Analytics Framework.';

  %if "&lsaf_validateddataset" EQ "" %then %let lsaf_validateddataset=work.lsafqueryauditentries;
  %let macName = %sysfunc(lowcase(&SYSMACRONAME));
  
  %lsaf_query(lsaf_callingMacro=&macName,
              lsaf_queryMethod=%str(executeAuditEntryQuery),
              lsaf_returnPathVar=%str(_lsafauditentriesexportpath_),
              lsaf_querydataset=&lsaf_querydataset,
              lsaf_validateddataset=&lsaf_validateddataset,
              lsaf_exportlocation=&lsaf_exportlocation,
              lsaf_exportpath=&lsaf_exportpath,
              lsaf_overwrite=&lsaf_overwrite,
              lsaf_enableversioning=&lsaf_enableversioning,
              lsaf_versiontype=&lsaf_versiontype,
              lsaf_customversion=&lsaf_customversion,
              lsaf_comment=&lsaf_comment
  );

%mend lsaf_queryauditentries;
