%* lsaf_removemember                                                                               *;
%*                                                                                                 *;
%* Removes a user or a group as a member of the specified context.                                 *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_REMOVEMEMBER(LSAF_PATH=lsaf-path, LSAF_MEMBER=lsaf-member-identifier                      *;
%* <, LSAF_TYPE=lsaf-type, LSAF_GROUP_CONTEXT=lsaf-group-context>);                                *;
%*                                                                                                 *;
%* @macvar _lsafRC_  The return code.                                                              *;
%* @macvar _lsafMsg_ The return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - Required - The path of the context from which to remove the member.          *;
%* @param lsaf_member - Required - The ID of the user or the name of the group.                    *;
%* @param lsaf_type - Conditional - The type of the member. Valid values (case-insensitive):       *;
%*            - USER (Default)                                                                     *;
%*            - GROUP                                                                              *;
%* @param lsaf_group_context - Conditional - The context in which the group to remove is defined,  *;
%*            specified as a path (such as /SAS). This parameter is required, if LSAF_TYPE=GROUP.  *;
%*            Otherwise, it is ignored.                                                            *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.2                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2013-08-30  updates for standardization and coding conventions                         *;
%* @history 2016-08-29  rebrand as LSAF                                                            *;
%* @history 2019-01-15  updated for LSAF 5.1 functionality                                         *;
%*                                                                                                 *;

%macro lsaf_removemember(
  lsaf_path=,
  lsaf_member=,
  lsaf_type=USER,
  lsaf_group_context=
  ) /des='Remove a member from the context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/MembershipService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("removeMember", "%bquote(&lsaf_path)", "%bquote(&lsaf_type)", "%bquote(&lsaf_member)", "%bquote(&lsaf_group_context)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      if upcase("&lsaf_type")="GROUP"
        then logMsg=" The &lsaf_type &lsaf_group_context &lsaf_member has been removed from &lsaf_path.." ;
        else logMsg=" The &lsaf_type &lsaf_member has been removed from &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      if upcase("&lsaf_type")="GROUP"
        then logMsg=" Cannot remove &lsaf_type &lsaf_group_context &lsaf_member from &lsaf_path.." ;
        else logMsg=" Cannot remove &lsaf_type &lsaf_member from &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_removemember;
