%* lsaf_scheduleexistsbyname                                                                *;
%*                                                                                          *;
%* Indicates whether a schedule with the specified name exists.                             *;
%* NOTE: Schedule names are not unique. If a schedule with the specified name exists,       *;
%* this macro returns True.                                                                 *;
%*                                                                                          *;
%* The SAS macro variable _lsafScheduleExistsByName_ is set to a value of 1, if the         *;
%* schedule exists, and 0, when it does not. In the event of an incorrect call to the macro *;
%* that results in an error, the return value of _lsafScheduleExistsByName_ is set to -1.   *;
%*                                                                                          *;
%* The macro returns False, if the user does not have access to the schedule. Access to     *;
%* schedules is controlled by the global privileges PRIVILEGE_MANAGE_SCHEDULES and          *;
%* PRIVILEGE_MANAGE_ALL_SCHEDULES.                                                          *;
%*                                                                                          *;
%* Syntax:                                                                                  *;
%* %LSAF_SCHEDULEEXISTSBYNAME(LSAF_NAME=lsaf-name);                                         *;
%*                                                                                          *;
%* @macvar _lsafRC_  The return code.                                                       *;
%* @macvar _lsafMsg_ The return message.                                                    *;
%* @macvar _lsafScheduleExistsByName_ Indicates whether the schedule exists.                *;
%*                                                                                          *;
%* @param lsaf_name - Required -  The name of the schedule.                                 *;
%*                                                                                          *;
%* @macrotype LSAF                                                                          *;
%* @since 2.3                                                                               *;
%* @exposure external                                                                       *;
%*                                                                                          *;
%* @history 2010-10-01  initial code                                                        *;
%*                                                                                          *;
%*                                                                                          *;

%macro lsaf_scheduleexistsbyname(
  lsaf_name=
  ) /des='Does the schedule exist by the specified name?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getproductname();

  %global _lsafMsg_         
          _lsafRC_
          _lsafScheduleExistsByName_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib exists          length=3;
    attrib logMsgPrimary   length=$70;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/ScheduleService");
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("scheduleExistsByName", "%bquote(&lsaf_name)", results, exists);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the schedule exists by name: &lsaf_name." ;
      exists=-1;
    end;
    else
    do;
      logMsgPrimary=logMsgPrefix;
      if (exists) then
      do;
        logMsg=" The schedule exists with name &lsaf_name." ;
      end;
      else
      do;
        logMsg=" The schedule does not exist with name &lsaf_name." ;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafScheduleExistsByName_= " exists;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafScheduleExistsByName_", exists) ;

  run;

%mend lsaf_scheduleexistsbyname ;
