%* lsaf_updateowner                                                                              *;
%*                                                                                               *;
%* Updates the current owner of an item in the SAS Life Science Analytics Framework repository.  *;
%*                                                                                               *;
%* Syntax for files:                                                                             *;
%* %LSAF_UPDATEOWNER(LSAF_PATH=lsaf-path, LSAF_USERID=lsaf-userid);                              *;
%*                                                                                               *;
%* Syntax for containers:                                                                        *;
%* %LSAF_UPDATEOWNER(LSAF_PATH=lsaf-path, LSAF_USERID=lsaf-userid, LSAF_ACLTYPE=lsaf-acl-type);  *;
%*                                                                                               *;
%* @macvar _lsafRC_  The return code.                                                            *;
%* @macvar _lsafMsg_ The return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - Required - The path of the item in the SAS Life Science Analytics          *;
%*            Framework repository.                                                              *;
%* @param lsaf_userid - Required - The user ID for the account to be the new owner. To set the   *;
%*            owner for the DEFAULT permissions to be the creator of the item, specify the value *;
%*            <creator> as the value for lsaf-userid.                                            *;
%* @param lsaf_aclType - Conditional - The type of permission owner to set. The valid values     *;
%*            are DEFAULT and CURRENT (case-insensitive). Files have only CURRENT permissions.   *;
%*            Therefore, a value is not required when lsaf_path is a file. However, if a value   *;
%*            other than CURRENT or blank is entered for a file, the operation fails.            *;
%*            For container items, the value must be specified.                                  *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%* @history 2019-02-05  update to 2.1                                                            *;
%*                                                                                               *;

%macro lsaf_updateowner(
  lsaf_path=,
  lsaf_aclType=,
  lsaf_userid=
  ) /des='Change the owner of an item.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsgPrimary   length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/lsaf/macro/service/AclService") ;
    declare javaobj results("com/sas/lsaf/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("updateOwner", "%bquote(&lsaf_path)", "%bquote(&lsaf_aclType)", "%bquote(&lsaf_userid)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    %let acltypestring=;
    %if (%str(&lsaf_aclType) ne %str()) 
      %then %let acltypestring=%str(of &lsaf_aclType);
    if (returnCode eq 0) then
    do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" The owner &acltypestring was changed to &lsaf_userid for &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot change the owner &acltypestring to &lsaf_userid for &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrimary " " logMsg;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_updateowner;
