%* lsaf_updatepfsetupsignallocs                                                                   *;
%*                                                                                                *;
%* Updates the locations for process flow signal elements using a data set as input.              *;
%*                                                                                                *;
%* The locations for multiple signal elements can be modified using a single data set.            *;
%* Each location is represented as a single row in the input data set.                            *;
%*                                                                                                *;
%* The output data set from the lsaf_getpfsetupsignallocs macro can be used to determine the      *;
%* locations that are currently set on the signal element.                                        *;
%* The output data set from the lsaf_getpfsetupelements can be used to determine the signal       *;
%* elements for a process flow.                                                                   *;
%*                                                                                                *;
%* The signal locations that are represented in the input data set replace the list of locations  *;
%* currently saved on the signal element. A single row where LOCATION is blank result in          *;
%* clearing all signal locations for the specified element.                                       *;
%*                                                                                                *;
%* Failure and Message Handling:                                                                  *;
%* The macro performs a series of validation procedures to determine whether the input data set   *;
%* has the necessary structure for processing. If any of these validation procedures fail, an     *;
%* appropriate failure message is written to the SAS log file and the macro stops additional      *;
%* processing.                                                                                    *;
%*                                                                                                *;
%* Signal locations must adhere to the valid format of a SAS Life Science Analytics Framework     *;
%* repository item with the exception of container paths, which must end in slash star "/*".      *;
%* If an invalid value is specified for a location, the updates are aborted and the               *;
%* appropriate error message is written to the SAS log file.                                      *;
%*                                                                                                *;
%* In addition to other modifications, the macro sorts the input data set and stores the modified *;
%* data in a temporary data set called WORK.__SASMACRO_SIGNALLOCATIONSTEMP__. In the event        *;
%* that a failure occurs during processing, this temporary data set is retained as a reference.   *;
%* If all updates are performed successfully, this data set is deleted from the work library.     *;
%* A message is written to the SAS log file, reporting whether the data set was retained or not.  *;
%* Regardless of success or failure of previous runs, this data set is deleted at the             *;
%* beginning of each new run of the macro. The data values and sort order of the original input   *;
%* data set are maintained.                                                                       *;
%*                                                                                                *;
%* All records that pertain to the same signal element are processed as a single transaction by   *;
%* the macro. Messages are written to the SAS log file as each signal element is processed. If the*;
%* update of a signal element fails, the macro stops processing and a failure message is          *;
%* written to the SAS log file. Successful updates to signal elements made prior to the           *;
%* observation that contains the failure are maintained in SAS Life Science Analytics Framework.  *;
%*                                                                                                *;
%* If duplicate valid locations are provided for the same element, only the first value is        *;
%* maintained.                                                                                    *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_UPDATEPFSETUOSIGNALLOCS(SAS_DSNAME=SAS-data-set);                                        *;
%*                                                                                                *;
%* @macvar _lsafRC_  The return code.                                                             *;
%* @macvar _lsafMsg_ The return message.                                                          *;
%*                                                                                                *;
%* @param sas_dsname - Required - The name of a SAS data set that contains the locations for the  *;
%*            signal elements to modify, specified as libref.dataset. The data set must contain   *;
%*            at least the variables that are listed below.                                       *;
%*            Additional variables are ignored.                                                   *;
%*            - processFlowPath: The path of the context in which the process flow is located.    *;
%*            - processFlowName: The name of the process flow.                                    *;
%*            - elementID: The ID for the element applicable for the property.                    *;
%*            - location: The path for the signal location.                                       *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.5                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2017-01-13  original coding                                                           *;
%* @history 2019-02-12  update to 2.1                                                             *;
%* @history 2019-03-18  renamed from lsaf_updatepfsignallocations to lsaf_updatepfsetupsignallocs *;
%*                                                                                                *;

%macro lsaf_updatepfsetupsignallocs(
  sas_dsname=
  ) /des='Update the locations for process flow signal elements.';

  %*************************************************************;
  %* Turn off the notes generated for all the pre-processing.  *;
  %* Save original option setting so it can be reset properly. *;
  %*************************************************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local logMsgPrefix
         logMsgPrimary
         macroName
         tempDsMsg
         _tempUPDATE_tempDS_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let macroName=%sysfunc(lowcase(&SYSMACRONAME));

  %***************************************************************************;
  %* Cleanup from any previous runs that may have left temp data set behind. *;
  %***************************************************************************;
  %let _tempUPDATE_tempDS_=%str(__SASMACRO_SIGNALLOCATIONSTEMP__);

  %if (%sysfunc(exist(&_tempUPDATE_tempDS_)) > 0) %then
  %do;
    %put NOTE: &macroName: Deleting temporary data set &_tempUPDATE_tempDS_ before processing.;
    proc datasets library=work nolist;
      delete &_tempUPDATE_tempDS_;
    quit; run;
  %end;

  %***************************************************************************;
  %* Set local macro variables for processing.                               *;
  %*                                                                         *;
  %* setting a new macro variable for operation success because there are    *;
  %* operations in updateACL that could set _lsafRC_ to something other than *;
  %* 0 but still not be a failure.                                           *;
  %***************************************************************************;
  %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
  %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );
  %let tempDsMsg=%str(Temporary sorted data set work.&_tempUPDATE_tempDS_ will not be created.);

  %***************************************************************************;
  %* Validate the input data set                                             *;
  %* sets global macro variable _lsafRC_=0 if validation successful          *;
  %***************************************************************************;
  %lsaf_validate_sasdataset(callingMacro=&macroName,
        sas_dsn=&sas_dsname,
        required_variables=%str(processFlowPath processFlowName elementID location)
        );
  %if (&_lsafRC_ ne 0) %then
  %do;
    options &notesOption;
    %put &logMsgPrimary _lsafMsg_= &_lsafMsg_ ;
    %put &logMsgPrefix &macroName ;
    %put &logMsgPrefix Input data set = &sas_dsname;
    %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
    %put &logMsgPrefix &tempDsMsg ;
    %put;
    %return;
  %end;

  %***************************************************************************;
  %* If data set is valid, process it.                                       *;
  %***************************************************************************;
  %put NOTE: &macroName: Processing data set &sas_dsname;

  %************************;
  %* Sort the data        *;
  %************************;
  proc sort data=&sas_dsname out=&_tempUPDATE_tempDS_;
    by processFlowPath processFlowName elementID;
  run;

  option notes;

  %**********************************************;
  %* Process the records                        *;
  %**********************************************;
  data _null_;
    attrib returnCode    length=8
           message       length=$200
           updateCount   length=8
    ;

    set &_tempUPDATE_tempDS_ end=eof;
    by processFlowPath processFlowName elementID;

    %***********************************;
    %* Declare the java objects, once  *;
    %***********************************;
    if (_n_ = 1) then
    do;
      declare javaobj locs("com/sas/lsaf/macro/results/support/SasStrings");
      declare javaobj srv("com/sas/lsaf/macro/service/ProcessFlowSetupService");
      declare javaobj results("com/sas/lsaf/macro/results/Result");

      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * " %sysfunc(lowcase("&SYSMACRONAME"));
      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Input data set = &sas_dsname";
    end;

    if (first.elementID) then
    do;
      locs.callVoidMethod("clearList");
      %**********************************************;
      %* Do not want to add a blank location if the *;
      %* user is trying to clear out the list.      *;
      %**********************************************;
      if (last.elementID AND kstrip(location)="") 
        then GOTO DO_UPDATE;
    end;

    %***********************************;
    %* Create a new locations list     *;
    %***********************************;
    locs.callVoidMethod("addValue", kstrip(location));

    if (last.elementID) then GOTO DO_UPDATE;

    RETURN;

    DO_UPDATE:
      %***********************************;
      %* Call the update                 *;
      %***********************************;
      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Processing element " elementID " in process flow " processFlowName " in "
          processFlowPath;
      srv.callStaticVoidMethod("updateSignalLocations", kstrip(processflowpath), kstrip(processflowname), kstrip(elementID),
          locs, results);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      if (returnCode ne 0) then
      do;
        put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * Unable to process " elementID;
        put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * _lsafMsg_ = " message;
        put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * _lsafRC_ = " returnCode;
        put;
        GOTO ENDPROCESS;
      end;
      else updateCount+1;

      if (eof) then GOTO ENDPROCESS;

    RETURN;

    ENDPROCESS:
    put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Successfully processed " updateCount "signal elements";
    put " ";

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

    STOP;

  run;

  %************************************************************;
  %* Cleanup temporary data set if the operation did not fail *;
  %************************************************************;
  %if (&_lsafRC_ eq 0) %then
  %do;
    %if (%sysfunc(exist(&_tempUPDATE_tempDS_))>0) %then
    %do;
      options notes;
      %put NOTE: &_LSAF_PRODUCT_NAME_ Macro: * %sysfunc(lowcase(&SYSMACRONAME)) is deleting temporary data set &_tempUPDATE_tempDS_;
      options nonotes;
      proc datasets library=work nolist;
        delete &_tempUPDATE_tempDS_;
        quit; run;
    %end;
  %end;
  %else %put WARNING: &_LSAF_PRODUCT_NAME_ Macro: * Temporary sorted data set work.&_tempUPDATE_tempDS_ is not being deleted.;

  options &notesOption;

%mend lsaf_updatepfsetupsignallocs;
