%* lsaf_updatestudydictionaries                                                                     *;
%*                                                                                                  *;
%* Updates the controlled terminology dictionaries for a study using a SAS data set as input.       *;
%* The order of the dictionary identifiers in the input data set dictates the order in which the    *;
%* dictionaries are associated to the study. This order is important and determines the precedence  *;
%* of duplicate code lists in the study. For example, if multiple dictionaries have the same        *;
%* named code list, only the code list from the first dictionary is associated with the study.      *;
%* All other duplicates are ignored.                                                                *;
%*                                                                                                  *;
%* Each dictionary identifier is represented as a single row in the input data set.                 *;
%*                                                                                                  *;
%* Use the output data set from the lsaf_getstudydictionaries macro to determine the                *;
%* controlled terminology dictionaries that are currently associated with a study. Consider running *;
%* this macro first to have a record of the current associated controlled terminology dictionaries. *;
%* If there are any issues with the updates, this data set can be used to reinstate the dictionary  *;
%* associations prior to running the updates.                                                       *;
%*                                                                                                  *;
%* The output data set from the lsaf_getdictionariesbystatus (published) can be used to determine   *;
%* dictionary identifiers that are available to be associated with the study.                       *;
%*                                                                                                  *;
%* The dictionary identifiers that are represented in the input data set replace the list of        *;
%* dictionaries that are currently associated with the study.                                       *;
%*                                                                                                  *;
%* Failure and Message Handling:                                                                    *;
%* The macro performs a series of validation procedures to determine whether the input data set     *;
%* has the necessary structure for processing. If any of these validation procedures fail, an       *;
%* appropriate failure message is written to the SAS log file and the macro stops additional        *;
%* processing.                                                                                      *;
%*                                                                                                  *;
%* If an invalid value is specified for a dictionary identifier, the updates are aborted and the    *;
%* appropriate error message is written to the SAS log file.                                        *;
%*                                                                                                  *;
%* If duplicate valid dictionary identifiers are provided for the study, only the first value is    *;
%* is maintained.                                                                                   *;
%*                                                                                                  *;
%* Syntax:                                                                                          *;
%* %LSAF_UPDATESTUDYDICTIONARIES(LSAF_STUDYID=lsaf-studyid, SAS_DSNAME=SAS-data-set);               *;
%*                                                                                                  *;
%* @macvar _lsafRC_  The return code.                                                               *;
%* @macvar _lsafMsg_ The return message.                                                            *;
%*                                                                                                  *;
%* @param lsaf_studyid - Required - The study item identifier. The study item identifier can be     *;
%*            retrieved by calling the lsaf_getstudyid macro.                                       *;
%* @param sas_dsname - Required - The name of a SAS data set that contains the dictionary           *;
%*            identifiers for the study to update, specified as libref.dataset. The data set must   *;
%*            contain at least the variable that is listed below. Additional variables are ignored. *;
%*            - id: The identifier for the controlled terminology dictionary. The identifier        *;
%*              can be retrieved by calling the lsaf_getdictionariesbystatus macro with the         *;
%*              PUBLISHED option.                                                                   *;
%* @macrotype LSAF                                                                                  *;
%* @since 2.2                                                                                       *;
%* @exposure external                                                                               *;
%*                                                                                                  *;
%* @history 2019-08-14  initial coding                                                              *;
%*                                                                                                  *;

%macro lsaf_updatestudydictionaries(lsaf_studyid=,  
  sas_dsname=
  ) /des='Update the controlled terminology dictionaries for a SAS Life Science Analytics Framework study.';

  %*************************************************************;
  %* Turn off the notes generated for all the pre-processing.  *;
  %* Save original option setting so it can be reset properly. *;
  %*************************************************************;
  %let notesOption=%sysfunc(getoption(notes));

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local logMsgPrefix
         logMsgPrimary
         macroName
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
  
  %let logMsgPrimary=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
  %let logMsgPrefix=%str(NOTE: &_LSAF_PRODUCT_NAME_ Macro: );

  %***************************************************************************;
  %* Validate the input data set                                             *;
  %* sets global macro variable _lsafRC_=0 if validation successful          *;
  %***************************************************************************;
  %lsaf_validate_sasdataset(callingMacro=&macroName,
        sas_dsn=&sas_dsname,
        required_variables=%str(id)
        );
  %if (&_lsafRC_ ne 0) %then
  %do;
    options &notesOption;
    %put &logMsgPrimary _lsafMsg_= &_lsafMsg_ ;
    %put &logMsgPrefix &macroName ;
    %put &logMsgPrefix Input data set = &sas_dsname;
    %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
    %put;
    %return;
  %end;

  %***************************************************************************;
  %* If data set is valid, process it.                                       *;
  %***************************************************************************;
  %put NOTE: &macroName: Processing data set &sas_dsname;

  option notes;

  %***********************;
  %* Process the records *;
  %***********************;

  data _null_;
    attrib returnCode    length=8
           message       length=$200
           logMsgPrefix  length=$70
           logMsgPrimary length=$70
           logMsg        length=$300
    ;
    set &sas_dsname end=eof;

    %**********************************;
    %* Declare the java object, once  *;
    %**********************************;
    if (_n_ = 1) then
    do;
      declare javaobj dictionaryIds("com/sas/lsaf/macro/results/support/SasStrings");
    end;

    %*************************************;
    %* Populate the dictionaryIds list   *; 
    %*************************************;
    dictionaryIds.callVoidMethod("addValue", kstrip(id));

    if eof then 
    do;
      %***********************************;
      %* Call the update                 *;
      %***********************************; 
      declare javaobj srv("com/sas/lsaf/macro/service/StudyService");
      declare javaobj results("com/sas/lsaf/macro/results/Result");
      srv.callStaticVoidMethod("setStudyDictionaries", "%bquote(&lsaf_studyid)", dictionaryIds, results);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (returnCode ne 0) then
      do;
        logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
        logMsg=" Cannot update controlled terminology dictionaries.";
      end;
      else
      do;
        logMsgPrimary=logMsgPrefix;
        logMsg=" Update of controlled terminology dictionaries was performed.";
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrimary " " logMsg;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrefix " _lsafMsg_= " message ;
      put logMsgPrefix " _lsafRC_= " returnCode ;
      put logMsgPrefix " Input data set = &sas_dsname";
      put;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);
    end;
  run;

%mend lsaf_updatestudydictionaries;