%* lsaf_validate_sasdataset.sas                                                        *;
%*                                                                                     *;
%* Validates the data set that is passed by the user to the SAS macros. The tests      *;
%* are:                                                                                *;
%* - The data set has a valid SAS name.                                                *;
%* - The data set exists.                                                              *;
%* - Optional (Default) - The data set is not empty (has observations).                *;
%* - The data set includes the required variables.                                     *;
%* - The required variables are of the correct variable type (character or numeric)    *;                                                       *;
%*                                                                                     *;
%* Syntax:                                                                             *;
%* %LSAF_VALIDATE_SASDATASET(SAS_DSN=SAS-data-set                                      *;
%*            <, CALLINGMACRO=calling-macro, REQUIRED_VARIABLES=required-var-list,     *;
%*            REQUIRED_NUMERIC_VARIABLES=required-numeric-list>,                       *;
%*            ALLOW_EMPTY_DATASETS=allow-empty-datasets);                              *;
%*                                                                                     *;
%* @macvar _lsafRC_  The return code.                                                  *;
%* @macvar _lsafMsg_ The return message.                                               *;
%*                                                                                     *;
%* @param sas_dsn - Required - The name of the data set to validate.                   *;
%* @param callingMacro - Optional - The macro that called this macro.                  *;
%* @param required_variables - Optional - A space-delimited list of data set variables *;
%*             that are required to be present in the data set.                        *;
%* @param required_numeric_variables - Optional - A space-delimited list of the        *;
%*             required variables that must be of type numeric. If a required          *;
%*             variable is not included in this list, it is validated as a             *;
%*             character.                                                              *;
%* @param allow_empty_datasets - Optional - Indicates whether an error is returned     *;
%*             when the input data set has 0 obs. The default is 0 (do not allow       *;
%*             0 obs).                                                                 *;
%*                                                                                     *;
%* @macrotype LSAF                                                                     *;
%* @since 1.2                                                                          *;
%* @exposure internal                                                                  *;
%*                                                                                     *;
%* @history 2013-08-30  updates for standardization and coding conventions.            *;
%* @history 2014-03-28  decoupled from updateacls.                                     *;
%* @history 2016-08-26  rebrand as LSAF                                                *;
%* @history 2016-11-07  added parameter allow_empty_datasets                           *;
%* @history 2017-01-09  updated documentation for allow_empty_datasets                 *;
%*                                                                                     *;

%macro lsaf_validate_sasdataset(
   sas_dsn=,
   callingMacro=,
   required_variables=,
   required_numeric_variables=%str(),
   allow_empty_datasets=0
   ) / des='Validates a SAS data set.';

   %put NOTE: &callingMacro.: Validating data set &sas_dsn;

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %global _lsafMsg_
           _lsafRC_
   ;

   %let _lsafRC_=%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %****************************************;
   %* VERIFY: data set name                *;
   %****************************************;
   %if (%qcmpres(&sas_dsn) eq %str()) %then
   %do;
      %let _lsafMsg_= %str(Input data set name was not provided.);
      %return;
   %end;

   %let validdsn=0;
   %let dsnmess=;

   data _null_;
      length message $45 datasetName2 $300;
      datasetName=kstrip("&sas_dsn");
      datasetName2="INVALID";
      error=0;
      %************************************************************************;
      %* Check for 2 level data set name (ie work.ABC).                       *;
      %* If 0 then one level name (ie ABC).                                   *;
      %* If 1 then it is a two level name (ie work.ABC).                      *;
      %* If 2 or more periods then an invalid data set name (ie work.xyz.ABC) *;
      %************************************************************************;
      period=countc(datasetName,".");
      libname_size=kindexc(datasetName,".");
      %***************************************************;
      %* Check to make sure we have a complete two level *;
      %* data set name (ie .ABC and work. are invalid).  *;
      %***************************************************;
      if libname_size=1 or libname_size=length(datasetName)
      then 
      do;
        error=1;
        message=" Missing either LIBNAME or data set.";
      end;
      else
      do;
        if period ne 0 then 
        do;
          select (period);
            when (1)
            do;
              %*******************************************************;
              %* check if libname statement longer than 8 characters *;
              %*******************************************************;
              if libname_size le 9 
              then datasetName2=(ksubstr(datasetName,libname_size+1));
              else 
              do;
                message=" Libname longer that 8 characters.";
                error=1;
              end;
            end;
            otherwise error=1;
          end;
        end;
        else datasetName2=datasetName;
      end;
      firstchar=ksubstr(kleft(datasetName2), 1, 1);
      if ((length(datasetName2) > 32) or (anydigit(firstchar)) or (error=1)) then
      do;
        if length(datasetName2) > 32 then message=" Data set name longer than 32 characters.";
        if anydigit(firstchar) then message=" Data set name must start with a chracter.";
        call symput("validdsn", put(1, best.));
        call symput("dsnmess", message);
      end;
   run;

   %if (&validdsn eq 1) %then
   %do;
     %let _lsafMsg_= %str(Input data set name is not valid. &dsnmess);
     %return;
   %end;

   %****************************************;
   %* VERIFY: data set exists              *;
   %****************************************;
   %if (%sysfunc(exist(&sas_dsn))=0) %then
   %do;
     %let _lsafMsg_= %str(Input data set &sas_dsn does not exist. );
     %return;
   %end;

   %local dsid;
   %let DSID=%sysfunc(OPEN(&sas_dsn., IN));

   %****************************************;
   %* VERIFY: data set has observations    *;
   %****************************************;
   %if (&allow_empty_datasets EQ 0) %then
   %do;
     %local nobs dsrc;
     %let RECORDS=%sysfunc(ATTRN(&DSID, NLOBS));
     %if (%eval(&RECORDS eq 0)) %then
     %do;
       %let _lsafMsg_= %str(Input data set &sas_dsn has no observations.);
       %let dsrc=%sysfunc(CLOSE(&DSID));
       %return;
     %end;
   %end;

   %********************************************************************;
   %* VERIFY: required parameters exist and are of the correct type.
   %********************************************************************;
   %if (%qcmpres(&required_variables) eq %str()) 
     %then %GOTO END_VALIDATION;

   %local varCnt varName varPos missingVars invalidVars DSINVALID;
   %let varCnt=1;
   %let varName=_null_;
   %let missingVars=%str();
   %let invalidCharVars=%str();
   %let invalidNumVars=%str();
   %let DSINVALID=0;

   %do %until (&varName=%str());
     %let varName=%scan(&required_variables, &varCnt);
     %if (&varName eq %str()) 
       %then %goto CONTINUE;
     %let varPos=%sysfunc(varnum(&DSID,&varName));

     %************************************;
     %* variable is missing              *;
     %************************************;
     %if (%eval(&varPos < 1)) %then
     %do;
       %let missingVars=%str(%trim(&missingVars) %trim(&varName));
       %let DSINVALID=1;
       %goto CONTINUE;
     %end;

     %let actualVarType = %sysfunc( vartype(&DSID, &varPos) );

     %if (%index(&required_numeric_variables, &varName) > 0) %then
     %do;
       %******************************************;
       %* variable that should be NUMERIC is not *;
       %******************************************;
       %if (&actualVarType ne N) %then
       %do;
         %let invalidNumVars=%str(%trim(&invalidNumVars) %trim(&varName));
         %let DSINVALID=1;
       %end;
     %end;
     %else %if (&actualVarType ne C) %then
     %do;
       %*********************************************;
       %*  variable that should be CHARACTER is not *;
       %*********************************************;
       %let invalidCharVars=%str(%trim(&invalidCharVars) %trim(&varName));
       %let DSINVALID=1;
     %end;

     %CONTINUE:
     %let varCnt=%eval(&varCnt+1);
   %end;

   %let rc=%sysfunc(CLOSE(&DSID));

   %if (&DSINVALID ne 0) %then
   %do;
     %**************************************************************************************;
     %* only setting one message so giving missing variables precedence over variable type *;
     %**************************************************************************************;
     %if (&missingVars ne %str()) %then
     %do;
       %let _lsafMsg_= %cmpres(Data set &sas_dsn is missing required variables: &missingVars);
       %put;
     %end;
     %else %if (&invalidNumVars ne %str()) %then
     %do;
       %let _lsafMsg_= %cmpres(Variables in data set &sas_dsn that must be numeric are not: &invalidNumVars);
     %end;
     %else %if (&invalidCharVars ne %str()) %then
     %do;
       %let _lsafMsg_= %cmpres(Variables in data set &sas_dsn that must be character are not: &invalidCharVars);
     %end;
     %return;
   %end;

   %END_VALIDATION:
   %let _lsafRC_=%str(0);
   %let _lsafMsg_=%str(Data set &sas_dsn is valid for &callingMacro..);

%mend lsaf_validate_sasdataset;
