%* lsaf_login                                                                                                   *;                                                                                                                                                                                                                   
%*                                                                                                              *;                                                                                                                                                                                                                   
%* Creates a connection to the SAS Life Science Analytics Framework instance.                                   *;                                                                                                                                                                                                                    
%*                                                                                                              *;                                                                                                                                                                                                                   
%* This macro is required only when running the macros from a PC SAS session. It must be called                 *;                                                                                                                                                                                                                   
%* before any other SAS Life Science Analytics Framework macro.                                                 *;                                                                                                                                                                                                                    
%*                                                                                                              *;                                                                                                                                                                                                                   
%* NOTE: This macro is supported only from a PC SAS session. Using this macro within an active                  *;                                                                                                                                                                                                                   
%* SAS Life Science Analytics Framework application session results in a failure.                               *;                                                                                                                                                                                                                    
%*                                                                                                              *;                                                                                                                                                                                                                   
%* Syntax:                                                                                                      *;                                                                                                                                                                                                                   
%* %LSAF_LOGIN(LSAF_URL=url, LSAF_USERID=user-id , LSAF_PASSWORD=password                                       *;                                                                                                                                                                                                                   
%*            <,PROXY_HOST=proxy-host, PROXY_PORT=proxy-port, PROXY_USERID=proxy-userid,                        *;
%*              PROXY_PASSWORD=proxy-pw>);                                                                      *;                                                                                                                                                                                                                   
%*                                                                                                              *;  
%* Examples:                                                                                                    *;                                                                                                                                                                                                                   
%*                                                                                                              *;                                                                                                                                                                                                                   
%* 1. The parameters that are passed to the login macro are often strings that contain special characters       *;                                                                                                                                                                                                                   
%*    that might cause problems when resolved by SAS. You will probably need to wrap each of the parameters     *;                                                                                                                                                                                                                   
%*    in a SAS function call to prevent any issues, such as:                                                    *;                                                                                                                                                                                                                   
%*    %lsaf_login(lsaf_url=%str(https://lsafinstance), lsaf_userid=%str(lsaf-user-id),                          *;                                                                                                                                                                                                                   
%*    lsaf_password=%str (lsafpassword ));                                                                      *;                                                                                                                                                                                                                   
%*                                                                                                              *;                                                                                                                                                                                                                   
%* 2. Certain special characters that are common in passwords (such as ampersands and percent signs) might      *;                                                                                                                                                                                                                   
%*    require quotation marks, such as:                                                                         *;                                                                                                                                                                                                                   
%*    %lsaf_login(lsaf_url=%str(https://lsafinstance), lsaf_userid=%str(lsaf-user-id),                          *;                                                                                                                                                                                                                   
%*    lsaf_password=%nrbquote(lsafpassword ));                                                                  *;                                                                                                                                                                                                                   
%*                                                                                                              *;                                                                                                                                                                                                                   
%* @macvar _lsafRC_  The return code.                                                                           *;                                                                                                                                                                                                                   
%* @macvar _lsafMsg_ The return message.                                                                        *;                                                                                                                                                                                                                   
%*                                                                                                              *;                                                                                                                                                                                                                   
%* @param lsaf_url - Required - The URL of the SAS Life Science Analytics Framework instance to use, specified  *;                                                                                                                                                                                                                    
%*             in the form https://lsafname.sas.com or http://lsafname.sas.com.                                 *;                                                                                                                                                                                                                   
%* @param lsaf_userid - Required - The SAS Life Science Analytics Framework ID for the user account.            *;                                                                                                                                                                                                                    
%* @param lsaf_password - Required - The SAS Life Science Analytics Framework password for the user account.    *;                                                                                                                                                                                                                    
%* @param proxy_host - Conditional - The name of the host computer, when the SAS Life Science Analytics         *;                                                                                                                                                                                                                    
%*             Framework system is accessed through a proxy server.                                             *;                                                                                                                                                                                                                   
%* @param proxy_port - Conditional - The port number that is required for the proxy host computer.              *;                                                                                                                                                                                                                   
%*             Required when a proxy_host is designated.                                                        *;                                                                                                                                                                                                                   
%* @param proxy_userid - Conditional - The user ID when the proxy computer requires authentication.             *;                                                                                                                                                                                                                   
%* @param proxy_password - Conditional - The password when the proxy computer requires authentication.          *;                                                                                                                                                                                                                   
%*                                                                                                              *;                                                                                                                                                                                                                   
%* @macrotype LSAF                                                                                              *;                                                                                                                                                                                                                   
%* @since 1.1                                                                                                   *;                                                                                                                                                                                                                   
%* @exposure external                                                                                           *;                                                                                                                                                                                                                   
%*                                                                                                              *;                                                                                                                                                                                                                   
%* @history 2013-06-26  version 1.2 update - trusted session                                                    *;                                                                                                                                                                                                                   
%* @history 2013-08-30  updates for standardization and coding conventions                                      *;                                                                                                                                                                                                                   
%* @history 2014-03-04  updates for proxy                                                                       *;                                                                                                                                                                                                                   
%* @history 2016-08-26  rebrand as LSAF                                                                         *; 
%* @history 2019-01-30  update to 2.1                                                                           *;
%* @history 2020-02-07  move to AuthenticationService                                                           *;  
%*                                                                                                              *;                                                                                                                                                                                                                   
                                                                                                                                                                                                                                                                                                                                     
%macro lsaf_login(                                                                                                                                                                                                                                                                                                                   
  lsaf_url=,                                                                                                                                                                                                                                                                                                                         
  lsaf_userid=,                                                                                                                                                                                                                                                                                                                      
  lsaf_password=,                                                                                                                                                                                                                                                                                                                    
  proxy_host=,                                                                                                                                                                                                                                                                                                                       
  proxy_port=,                                                                                                                                                                                                                                                                                                                       
  proxy_userid=,                                                                                                                                                                                                                                                                                                                     
  proxy_password=                                                                                                                                                                                                                                                                                                                    
  ) /des='Initialize connection to SAS Life Science Analytics Framework.';                                                                                                                                                                                                                                                           
                                                                                                                                                                                                                                                                                                                                     
  %****************************************;                                                                                                                                                                                                                                                                                         
  %* Initialize macro variables           *;                                                                                                                                                                                                                                                                                         
  %****************************************;                                                                                                                                                                                                                                                                                         
  %lsaf_getProductName();                                                                                                                                                                                                                                                                                                            
                                                                                                                                                                                                                                                                                                                                     
  %global _lsafMsg_                                                                                                                                                                                                                                                                                                                  
          _lsafRC_                                                                                                                                                                                                                                                                                                                   
  ;                                                                                                                                                                                                                                                                                                                                  
                                                                                                                                                                                                                                                                                                                                     
  %let _lsafRC_ =%str(-999);                                                                                                                                                                                                                                                                                                         
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);                                                                                                                                                                                                                                                
                                                                                                                                                                                                                                                                                                                                     
  %**********************************************;                                                                                                                                                                                                                                                                                   
  %* Start the datastep to call the java code   *;                                                                                                                                                                                                                                                                                   
  %**********************************************;                                                                                                                                                                                                                                                                                   
  data _null_;                                                                                                                                                                                                                                                                                                                       
    attrib returnCode      length=8;                                                                                                                                                                                                                                                                                                 
    attrib message         length=$200;                                                                                                                                                                                                                                                                                              
    attrib logMsgPrefix    length=$70;                                                                                                                                                                                                                                                                                               
    attrib logMsgPrimary   length=$70;                                                                                                                                                                                                                                                                                               
    attrib logMsg          length=$300;                                                                                                                                                                                                                                                                                              
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Declare the java objects        *;                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    declare javaobj srv("com/sas/lsaf/macro/service/AuthenticationService");                                                                                                                                                                                                                                                                 
    declare javaobj results("com/sas/lsaf/macro/results/Result");                                                                                                                                                                                                                                                                     
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Attempt the operation           *:                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    srv.callStaticVoidMethod("login", "%bquote(&lsaf_url)", "%bquote(&lsaf_userid)", "%bquote(&lsaf_password)",                                                                                                                                                                                                                      
         "%bquote(&proxy_host)", "%bquote(&proxy_port)", "%bquote(&proxy_userid)", "%bquote(&proxy_password)", results);                                                                                                                                                                                                             
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Retrieve the results            *;                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    results.callIntMethod("getReturnCode", returnCode);                                                                                                                                                                                                                                                                              
    results.callStringMethod("getReturnMessage", message);                                                                                                                                                                                                                                                                           
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Process the results             *;                                                                                                                                                                                                                                                                                            
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    if (returnCode ne 0) then                                                                                                                                                                                                                                                                                                        
    do;                                                                                                                                                                                                                                                                                                                              
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";                                                                                                                                                                                                                                                                       
      logMsg="Could not login to the &_LSAF_PRODUCT_NAME_ Server." ;                                                                                                                                                                                                                                                                 
    end;                                                                                                                                                                                                                                                                                                                             
    else                                                                                                                                                                                                                                                                                                                             
    do;                                                                                                                                                                                                                                                                                                                              
      logMsgPrimary=logMsgPrefix;                                                                                                                                                                                                                                                                           
      logMsg="&_LSAF_PRODUCT_NAME_ Session initialized.";                                                                                                                                                                                                                                                                            
    end;                                                                                                                                                                                                                                                                                                                             
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Print messages in log           *;                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    put;                                                                                                                                                                                                                                                                                                                             
    put logMsgPrimary " " logMsg;                                                                                                                                                                                                                                                                                                     
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));                                                                                                                                                                                                                                                                         
    put logMsgPrefix " url= &lsaf_url";                                                                                                                                                                                                                                                                                              
    put logMsgPrefix " userid= &lsaf_userid";                                                                                                                                                                                                                                                                                        
    if (klength(kstrip("&proxy_host")) > 0) then                                                                                                                                                                                                                                                                                     
    do;                                                                                                                                                                                                                                                                                                                              
       put logMsgPrefix " proxy host= &proxy_host";                                                                                                                                                                                                                                                                                  
       put logMsgPrefix " proxy port= &proxy_port";                                                                                                                                                                                                                                                                                  
       put logMsgPrefix " proxy_userid= &proxy_userid";                                                                                                                                                                                                                                                                              
    end;                                                                                                                                                                                                                                                                                                                             
    put logMsgPrefix " _lsafMsg_= " message;                                                                                                                                                                                                                                                                                         
    put logMsgPrefix " _lsafRC_= " returnCode;                                                                                                                                                                                                                                                                                       
    put;                                                                                                                                                                                                                                                                                                                             
                                                                                                                                                                                                                                                                                                                                     
    %***********************************;                                                                                                                                                                                                                                                                                            
    %* Set the macro variables         *;                                                                                                                                                                                                                                                                                            
    %***********************************;                                                                                                                                                                                                                                                                                            
    call symputx("_lsafRC_", returnCode);                                                                                                                                                                                                                                                                                            
    call symputx("_lsafMsg_", message);                                                                                                                                                                                                                                                                                              
                                                                                                                                                                                                                                                                                                                                     
  run;                                                                                                                                                                                                                                                                                                                               
                                                                                                                                                                                                                                                                                                                                     
%mend lsaf_login;                                                                                                                                                                                                                                                                                                                   
